@@ -1,5 +1,6 @@
 /*
 
+Copyright 2014 S. Razi Alavizadeh
 Copyright 2012-2014 Adam Reichold
 
 This file is part of qpdfview.
@@ -24,6 +25,8 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <QMenu>
 
+#include <QPair>
+
 class QFileInfo;
 
 #include "global.h"
@@ -38,16 +41,16 @@ class BookmarkMenu : public QMenu
 public:
     BookmarkMenu(const QFileInfo& fileInfo, QWidget* parent = 0);
 
+    inline QString absoluteFilePath() const { return menuAction()->data().toString(); }
+
     void addJumpToPageAction(int page, const QString& label);
     void removeJumpToPageAction(int page);
 
-    QString absoluteFilePath() const;
-    JumpList jumps() const;
-
 signals:
     void openTriggered(const QString& filePath);
     void openInNewTabTriggered(const QString& filePath);
     void jumpToPageTriggered(const QString& filePath, int page);
+    void removeBookmarkTriggered(const QString& filePath);
 
 protected slots:
     void on_open_triggered();
