@@ -1,5 +1,6 @@
 /*
 
+Copyright 2014 S. Razi Alavizadeh
 Copyright 2012-2014 Adam Reichold
 Copyright 2012 Michał Trybus
 
@@ -47,7 +48,9 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "settings.h"
 #include "documentview.h"
-#include "bookmarkmenu.h"
+#include "bookmarkmodel.h"
+
+#ifdef WITH_SQL
 
 namespace
 {
@@ -81,6 +84,8 @@ private:
 
 } // anonymous
 
+#endif // WITH_SQL
+
 namespace qpdfview
 {
 
@@ -145,7 +150,7 @@ void Database::restoreTabs()
         Transaction transaction(m_database);
 
         QSqlQuery query(m_database);
-        query.prepare("SELECT filePath,currentPage,continuousMode,layoutMode,scaleMode,scaleFactor,rotation FROM tabs_v2 WHERE instanceName==?");
+        query.prepare("SELECT filePath,currentPage,continuousMode,layoutMode,rightToLeftMode,scaleMode,scaleFactor,rotation FROM tabs_v3 WHERE instanceName==?");
 
         query.bindValue(0, instanceName());
 
@@ -162,9 +167,10 @@ void Database::restoreTabs()
             emit tabRestored(query.value(0).toString(),
                              static_cast< bool >(query.value(2).toUInt()),
                              static_cast< LayoutMode >(query.value(3).toUInt()),
-                             static_cast< ScaleMode >(query.value(4).toUInt()),
-                             query.value(5).toReal(),
-                             static_cast< Rotation >(query.value(6).toUInt()),
+                             query.value(4).toBool(),
+                             static_cast< ScaleMode >(query.value(5).toUInt()),
+                             query.value(6).toReal(),
+                             static_cast< Rotation >(query.value(7).toUInt()),
                              query.value(1).toInt());
         }
 
@@ -174,7 +180,7 @@ void Database::restoreTabs()
 #endif // WITH_SQL
 }
 
-void Database::saveTabs(const QList< const DocumentView* >& tabs)
+void Database::saveTabs(const QList< DocumentView* >& tabs)
 {
 #ifdef WITH_SQL
 
@@ -184,11 +190,36 @@ void Database::saveTabs(const QList< const DocumentView* >& tabs)
 
         QSqlQuery query(m_database);
 
-        if(Settings::instance()->mainWindow().restoreTabs())
-        {
-            query.prepare("DELETE FROM tabs_v2 WHERE instanceName==?");
+        query.prepare("DELETE FROM tabs_v3 WHERE instanceName==?");
 
-            query.bindValue(0, instanceName());
+        query.bindValue(0, instanceName());
+
+        query.exec();
+
+        if(!query.isActive())
+        {
+            qDebug() << query.lastError();
+            return;
+        }
+
+        query.prepare("INSERT INTO tabs_v3 "
+                      "(filePath,instanceName,currentPage,continuousMode,layoutMode,rightToLeftMode,scaleMode,scaleFactor,rotation)"
+                      " VALUES (?,?,?,?,?,?,?,?,?)");
+
+        foreach(const DocumentView* tab, tabs)
+        {
+            query.bindValue(0, tab->fileInfo().absoluteFilePath());
+            query.bindValue(1, instanceName());
+            query.bindValue(2, tab->currentPage());
+
+            query.bindValue(3, static_cast< uint >(tab->continuousMode()));
+            query.bindValue(4, static_cast< uint >(tab->layoutMode()));
+            query.bindValue(5, static_cast< uint >(tab->rightToLeftMode()));
+
+            query.bindValue(6, static_cast< uint >(tab->scaleMode()));
+            query.bindValue(7, tab->scaleFactor());
+
+            query.bindValue(8, static_cast< uint >(tab->rotation()));
 
             query.exec();
 
@@ -197,43 +228,6 @@ void Database::saveTabs(const QList< const DocumentView* >& tabs)
                 qDebug() << query.lastError();
                 return;
             }
-
-            query.prepare("INSERT INTO tabs_v2 "
-                          "(filePath,instanceName,currentPage,continuousMode,layoutMode,scaleMode,scaleFactor,rotation)"
-                          " VALUES (?,?,?,?,?,?,?,?)");
-
-            foreach(const DocumentView* tab, tabs)
-            {
-                query.bindValue(0, tab->fileInfo().absoluteFilePath());
-                query.bindValue(1, instanceName());
-                query.bindValue(2, tab->currentPage());
-
-                query.bindValue(3, static_cast< uint >(tab->continuousMode()));
-                query.bindValue(4, static_cast< uint >(tab->layoutMode()));
-
-                query.bindValue(5, static_cast< uint >(tab->scaleMode()));
-                query.bindValue(6, tab->scaleFactor());
-
-                query.bindValue(7, static_cast< uint >(tab->rotation()));
-
-                query.exec();
-
-                if(!query.isActive())
-                {
-                    qDebug() << query.lastError();
-                    return;
-                }
-            }
-        }
-        else
-        {
-            query.exec("DELETE FROM tabs_v2");
-
-            if(!query.isActive())
-            {
-                qDebug() << query.lastError();
-                return;
-            }
         }
 
         transaction.commit();
@@ -246,6 +240,29 @@ void Database::saveTabs(const QList< const DocumentView* >& tabs)
 #endif // WITH_SQL
 }
 
+void Database::clearTabs()
+{
+#ifdef WITH_SQL
+
+    if(m_database.isOpen())
+    {
+        Transaction transaction(m_database);
+
+        QSqlQuery query(m_database);
+        query.exec("DELETE FROM tabs_v3");
+
+        if(!query.isActive())
+        {
+            qDebug() << query.lastError();
+            return;
+        }
+
+        transaction.commit();
+    }
+
+#endif // WITH_SQL
+}
+
 void Database::restoreBookmarks()
 {
 #ifdef WITH_SQL
@@ -255,7 +272,7 @@ void Database::restoreBookmarks()
         Transaction transaction(m_database);
 
         QSqlQuery outerQuery(m_database);
-        outerQuery.exec("SELECT DISTINCT(filePath) FROM bookmarks_v2");
+        outerQuery.exec("SELECT DISTINCT(filePath) FROM bookmarks_v3");
 
         while(outerQuery.next())
         {
@@ -265,13 +282,14 @@ void Database::restoreBookmarks()
                 return;
             }
 
-            const QString filePath = outerQuery.value(0).toString();
-            QList< QPair< int, QString > > jumps;
+            const QString absoluteFilePath = outerQuery.value(0).toString();
+
+            BookmarkModel* model = BookmarkModel::fromPath(absoluteFilePath, true);
 
             QSqlQuery innerQuery(m_database);
-            innerQuery.prepare("SELECT page,label FROM bookmarks_v2 WHERE filePath==?");
+            innerQuery.prepare("SELECT page,label,comment,modified FROM bookmarks_v3 WHERE filePath==?");
 
-            innerQuery.bindValue(0, filePath);
+            innerQuery.bindValue(0, absoluteFilePath);
 
             innerQuery.exec();
 
@@ -285,11 +303,11 @@ void Database::restoreBookmarks()
 
                 const int page = innerQuery.value(0).toInt();
                 const QString label = innerQuery.value(1).toString();
+                const QString comment = innerQuery.value(2).toString();
+                const QDateTime modified = innerQuery.value(3).toDateTime();
 
-                jumps.append(qMakePair(page, label));
+                model->addBookmark(BookmarkItem(page, label, comment, modified));
             }
-
-            emit bookmarkRestored(filePath, jumps);
         }
 
         transaction.commit();
@@ -298,7 +316,7 @@ void Database::restoreBookmarks()
 #endif // WITH_SQL
 }
 
-void Database::saveBookmarks(const QList< const BookmarkMenu* >& bookmarks)
+void Database::saveBookmarks()
 {
 #ifdef WITH_SQL
 
@@ -307,7 +325,7 @@ void Database::saveBookmarks(const QList< const BookmarkMenu* >& bookmarks)
         Transaction transaction(m_database);
 
         QSqlQuery query(m_database);
-        query.exec("DELETE FROM bookmarks_v2");
+        query.exec("DELETE FROM bookmarks_v3");
 
         if(!query.isActive())
         {
@@ -317,17 +335,23 @@ void Database::saveBookmarks(const QList< const BookmarkMenu* >& bookmarks)
 
         if(Settings::instance()->mainWindow().restoreBookmarks())
         {
-            query.prepare("INSERT INTO bookmarks_v2 "
-                          "(filePath,page,label)"
-                          " VALUES (?,?,?)");
+            query.prepare("INSERT INTO bookmarks_v3 "
+                          "(filePath,page,label,comment,modified)"
+                          " VALUES (?,?,?,?,?)");
 
-            foreach(const BookmarkMenu* bookmark, bookmarks)
+            foreach(const QString& absoluteFilePath, BookmarkModel::knownPaths())
             {
-                foreach(const Jump jump, bookmark->jumps())
+                const BookmarkModel* model = BookmarkModel::fromPath(absoluteFilePath);
+
+                for(int row = 0, rowCount = model->rowCount(); row < rowCount; ++row)
                 {
-                    query.bindValue(0, bookmark->absoluteFilePath());
-                    query.bindValue(1, jump.first);
-                    query.bindValue(2, jump.second);
+                    const QModelIndex index = model->index(row);
+
+                    query.bindValue(0, absoluteFilePath);
+                    query.bindValue(1, index.data(BookmarkModel::PageRole));
+                    query.bindValue(2, index.data(BookmarkModel::LabelRole));
+                    query.bindValue(3, index.data(BookmarkModel::CommentRole));
+                    query.bindValue(4, index.data(BookmarkModel::ModifiedRole));
 
                     query.exec();
 
@@ -350,6 +374,29 @@ void Database::saveBookmarks(const QList< const BookmarkMenu* >& bookmarks)
 #endif // WITH_SQL
 }
 
+void Database::clearBookmarks()
+{
+#ifdef WITH_SQL
+
+    if(m_database.isOpen())
+    {
+        Transaction transaction(m_database);
+
+        QSqlQuery query(m_database);
+        query.exec("DELETE FROM bookmarks_v3");
+
+        if(!query.isActive())
+        {
+            qDebug() << query.lastError();
+            return;
+        }
+
+        transaction.commit();
+    }
+
+#endif // WITH_SQL
+}
+
 void Database::restorePerFileSettings(DocumentView* tab)
 {
 #ifdef WITH_SQL
@@ -359,7 +406,7 @@ void Database::restorePerFileSettings(DocumentView* tab)
         Transaction transaction(m_database);
 
         QSqlQuery query(m_database);
-        query.prepare("SELECT currentPage,continuousMode,layoutMode,rightToLeftMode,scaleMode,scaleFactor,rotation FROM perfilesettings_v2 WHERE filePath==?");
+        query.prepare("SELECT currentPage,continuousMode,layoutMode,rightToLeftMode,scaleMode,scaleFactor,rotation,firstPage FROM perfilesettings_v3 WHERE filePath==?");
 
         query.bindValue(0, QCryptographicHash::hash(tab->fileInfo().absoluteFilePath().toUtf8(), QCryptographicHash::Sha1).toBase64());
 
@@ -376,6 +423,8 @@ void Database::restorePerFileSettings(DocumentView* tab)
 
             tab->setRotation(static_cast< Rotation >(query.value(6).toUInt()));
 
+            tab->setFirstPage(query.value(7).toInt());
+
             tab->jumpToPage(query.value(0).toInt(), false);
         }
 
@@ -404,9 +453,9 @@ void Database::savePerFileSettings(const DocumentView* tab)
         Transaction transaction(m_database);
 
         QSqlQuery query(m_database);
-        query.prepare("INSERT OR REPLACE INTO perfilesettings_v2 "
-                      "(lastUsed,filePath,currentPage,continuousMode,layoutMode,rightToLeftMode,scaleMode,scaleFactor,rotation)"
-                      " VALUES (?,?,?,?,?,?,?,?,?)");
+        query.prepare("INSERT OR REPLACE INTO perfilesettings_v3 "
+                      "(lastUsed,filePath,currentPage,continuousMode,layoutMode,rightToLeftMode,scaleMode,scaleFactor,rotation,firstPage)"
+                      " VALUES (?,?,?,?,?,?,?,?,?,?)");
 
         query.bindValue(0, QDateTime::currentDateTime().toTime_t());
 
@@ -422,6 +471,8 @@ void Database::savePerFileSettings(const DocumentView* tab)
 
         query.bindValue(8, static_cast< uint >(tab->rotation()));
 
+        query.bindValue(9, tab->firstPage());
+
         query.exec();
 
         if(!query.isActive())
@@ -462,37 +513,66 @@ Database::Database(QObject* parent) : QObject(parent)
 
     if(m_database.isOpen())
     {
+        {
+            QSqlQuery query(m_database);
+            query.exec("PRAGMA synchronous = OFF");
+            query.exec("PRAGMA journal_mode = MEMORY");
+        }
+
         const QStringList tables = m_database.tables();
 
         // tabs
 
-        if(!tables.contains("tabs_v2"))
+        if(!tables.contains("tabs_v3"))
         {
-            if(prepareTabs_v2() && tables.contains("tabs_v1"))
+            if(prepareTabs_v3())
             {
-                migrateTabs_v1_v2();
+                if(tables.contains("tabs_v2"))
+                {
+                    migrateTabs_v2_v3();
+                }
+                else if(tables.contains("tabs_v1"))
+                {
+                    migrateTabs_v1_v3();
+                }
             }
         }
 
         // bookmarks
 
-        if(!tables.contains("bookmarks_v2"))
+        if(!tables.contains("bookmarks_v3"))
         {
-            if(prepareBookmarks_v2() && tables.contains("bookmarks_v1"))
+            if(prepareBookmarks_v3())
             {
-                migrateBookmarks_v1_v2();
+                if(tables.contains("bookmarks_v2"))
+                {
+                    migrateBookmarks_v2_v3();
+                }
+                else if(tables.contains("bookmarks_v1"))
+                {
+                    migrateBookmarks_v1_v3();
+                }
             }
         }
 
         // per-file settings
 
-        if(!tables.contains("perfilesettings_v2"))
+        if(!tables.contains("perfilesettings_v3"))
         {
-            if(preparePerFileSettings_v2() && tables.contains("perfilesettings_v1"))
+            if(preparePerFileSettings_v3())
             {
-                migratePerFileSettings_v1_v2();
+                if(tables.contains("perfilesettings_v2"))
+                {
+                    migratePerFileSettings_v2_v3();
+                }
+                else if(tables.contains("perfilesettings_v1"))
+                {
+                    migratePerFileSettings_v1_v3();
+                }
             }
         }
+
+        limitPerFileSettings();
     }
     else
     {
@@ -509,63 +589,17 @@ QString Database::instanceName()
 
 #ifdef WITH_SQL
 
-bool Database::prepareTabs_v2()
+bool Database::prepareTabs_v3()
 {
     Transaction transaction(m_database);
 
     QSqlQuery query(m_database);
-    query.exec("CREATE TABLE tabs_v2 "
+    query.exec("CREATE TABLE tabs_v3 "
                "(filePath TEXT"
                ",instanceName TEXT"
                ",currentPage INTEGER"
                ",continuousMode INTEGER"
                ",layoutMode INTEGER"
-               ",scaleMode INTEGER"
-               ",scaleFactor REAL"
-               ",rotation INTEGER)");
-
-    if(!query.isActive())
-    {
-        qDebug() << query.lastError();
-        return false;
-    }
-
-    transaction.commit();
-    return true;
-}
-
-bool Database::prepareBookmarks_v2()
-{
-    Transaction transaction(m_database);
-
-    QSqlQuery query(m_database);
-    query.exec("CREATE TABLE bookmarks_v2 "
-               "(filePath TEXT"
-               ",page INTEGER"
-               ",label TEXT)");
-
-    if(!query.isActive())
-    {
-        qDebug() << query.lastError();
-        return false;
-    }
-
-    transaction.commit();
-    return true;
-}
-
-bool Database::preparePerFileSettings_v2()
-{
-    Transaction transaction(m_database);
-
-    QSqlQuery query(m_database);
-
-    query.exec("CREATE TABLE perfilesettings_v2 "
-               "(lastUsed INTEGER"
-               ",filePath TEXT PRIMARY KEY"
-               ",currentPage INTEGER"
-               ",continuousMode INTEGER"
-               ",layoutMode INTEGER"
                ",rightToLeftMode INTEGER"
                ",scaleMode INTEGER"
                ",scaleFactor REAL"
@@ -577,14 +611,21 @@ bool Database::preparePerFileSettings_v2()
         return false;
     }
 
-    if(Settings::instance()->mainWindow().restorePerFileSettings())
-    {
-        query.exec("DELETE FROM perfilesettings_v2 WHERE filePath IN (SELECT filePath FROM perfilesettings_v2 ORDER BY lastUsed DESC LIMIT -1 OFFSET 1000)");
-    }
-    else
-    {
-        query.exec("DELETE FROM perfilesettings_v2");
-    }
+    transaction.commit();
+    return true;
+}
+
+bool Database::prepareBookmarks_v3()
+{
+    Transaction transaction(m_database);
+
+    QSqlQuery query(m_database);
+    query.exec("CREATE TABLE bookmarks_v3 "
+               "(filePath TEXT"
+               ",page INTEGER"
+               ",label TEXT"
+               ",comment TEXT"
+               ",modified DATETIME)");
 
     if(!query.isActive())
     {
@@ -596,13 +637,64 @@ bool Database::preparePerFileSettings_v2()
     return true;
 }
 
-void Database::migrateTabs_v1_v2()
+bool Database::preparePerFileSettings_v3()
 {
     Transaction transaction(m_database);
 
     QSqlQuery query(m_database);
-    query.prepare("INSERT INTO tabs_v2 "
-                  "SELECT filePath,?,currentPage,continuousMode,layoutMode,scaleMode,scaleFactor,rotation "
+
+    query.exec("CREATE TABLE perfilesettings_v3 "
+               "(lastUsed INTEGER"
+               ",filePath TEXT PRIMARY KEY"
+               ",currentPage INTEGER"
+               ",continuousMode INTEGER"
+               ",layoutMode INTEGER"
+               ",rightToLeftMode INTEGER"
+               ",scaleMode INTEGER"
+               ",scaleFactor REAL"
+               ",rotation INTEGER"
+               ",firstPage INTEGER)");
+
+    if(!query.isActive())
+    {
+        qDebug() << query.lastError();
+        return false;
+    }
+
+    transaction.commit();
+    return true;
+}
+
+void Database::migrateTabs_v2_v3()
+{
+    Transaction transaction(m_database);
+
+    QSqlQuery query(m_database);
+    query.prepare("INSERT INTO tabs_v3 "
+                  "SELECT filePath,instanceName,currentPage,continuousMode,layoutMode,0,scaleMode,scaleFactor,rotation "
+                  "FROM tabs_v2");
+
+    query.exec();
+
+    if(!query.isActive())
+    {
+        qDebug() << query.lastError();
+        return;
+    }
+
+    qWarning() << "Migrated tabs from v2 to v3, dropping v2.";
+    query.exec("DROP TABLE tabs_v2");
+
+    transaction.commit();
+}
+
+void Database::migrateTabs_v1_v3()
+{
+    Transaction transaction(m_database);
+
+    QSqlQuery query(m_database);
+    query.prepare("INSERT INTO tabs_v3 "
+                  "SELECT filePath,?,currentPage,continuousMode,layoutMode,0,scaleMode,scaleFactor,rotation "
                   "FROM tabs_v1");
 
     query.bindValue(0, instanceName());
@@ -615,13 +707,36 @@ void Database::migrateTabs_v1_v2()
         return;
     }
 
-    qWarning() << "Migrated tabs from v1 to v2, dropping v1.";
+    qWarning() << "Migrated tabs from v1 to v3, dropping v1.";
     query.exec("DROP TABLE tabs_v1");
 
     transaction.commit();
 }
 
-void Database::migrateBookmarks_v1_v2()
+void Database::migrateBookmarks_v2_v3()
+{
+    Transaction transaction(m_database);
+
+    QSqlQuery query(m_database);
+    query.prepare("INSERT INTO bookmarks_v3 "
+                  "SELECT filePath,page,label,'',datetime('now') "
+                  "FROM bookmarks_v2");
+
+    query.exec();
+
+    if(!query.isActive())
+    {
+        qDebug() << query.lastError();
+        return;
+    }
+
+    qWarning() << "Migrated bookmarks from v2 to v3, dropping v2.";
+    query.exec("DROP TABLE bookmarks_v2");
+
+    transaction.commit();
+}
+
+void Database::migrateBookmarks_v1_v3()
 {
     Transaction transaction(m_database);
 
@@ -637,9 +752,9 @@ void Database::migrateBookmarks_v1_v2()
         }
 
         QSqlQuery innerQuery(m_database);
-        innerQuery.prepare("INSERT INTO bookmarks_v2 "
-                           "(filePath,page,label)"
-                           " VALUES (?,?,?)");
+        innerQuery.prepare("INSERT INTO bookmarks_v3 "
+                           "(filePath,page,label,comment,modified)"
+                           " VALUES (?,?,?,'',datetime('now'))");
 
         innerQuery.bindValue(0, outerQuery.value(0));
 
@@ -664,19 +779,42 @@ void Database::migrateBookmarks_v1_v2()
         return;
     }
 
-    qWarning() << "Migrated bookmarks from v1 to v2, dropping v1.";
+    qWarning() << "Migrated bookmarks from v1 to v3, dropping v1.";
     outerQuery.exec("DROP TABLE bookmarks_v1");
 
     transaction.commit();
 }
 
-void Database::migratePerFileSettings_v1_v2()
+void Database::migratePerFileSettings_v2_v3()
 {
     Transaction transaction(m_database);
 
     QSqlQuery query(m_database);
-    query.prepare("INSERT INTO perfilesettings_v2 "
-                  "SELECT lastUsed,filePath,currentPage,continuousMode,layoutMode,0,scaleMode,scaleFactor,rotation "
+    query.prepare("INSERT INTO perfilesettings_v3 "
+                  "SELECT lastUsed,filePath,currentPage,continuousMode,layoutMode,rightToLeftMode,scaleMode,scaleFactor,rotation,-1 "
+                  "FROM perfilesettings_v2");
+
+    query.exec();
+
+    if(!query.isActive())
+    {
+        qDebug() << query.lastError();
+        return;
+    }
+
+    qWarning() << "Migrated per-file settings from v2 to v3, dropping v2.";
+    query.exec("DROP TABLE perfilesettings_v2");
+
+    transaction.commit();
+}
+
+void Database::migratePerFileSettings_v1_v3()
+{
+    Transaction transaction(m_database);
+
+    QSqlQuery query(m_database);
+    query.prepare("INSERT INTO perfilesettings_v3 "
+                  "SELECT lastUsed,filePath,currentPage,continuousMode,layoutMode,0,scaleMode,scaleFactor,rotation,-1 "
                   "FROM perfilesettings_v1");
 
     query.exec();
@@ -687,12 +825,36 @@ void Database::migratePerFileSettings_v1_v2()
         return;
     }
 
-    qWarning() << "Migrated per-file settings from v1 to v2, dropping v1.";
+    qWarning() << "Migrated per-file settings from v1 to v3, dropping v1.";
     query.exec("DROP TABLE perfilesettings_v1");
 
     transaction.commit();
 }
 
+void Database::limitPerFileSettings()
+{
+    Transaction transaction(m_database);
+
+    QSqlQuery query(m_database);
+
+    if(Settings::instance()->mainWindow().restorePerFileSettings())
+    {
+        query.exec("DELETE FROM perfilesettings_v3 WHERE filePath NOT IN (SELECT filePath FROM perfilesettings_v3 ORDER BY lastUsed DESC LIMIT 1000)");
+    }
+    else
+    {
+        query.exec("DELETE FROM perfilesettings_v3");
+    }
+
+    if(!query.isActive())
+    {
+        qDebug() << query.lastError();
+        return;
+    }
+
+    transaction.commit();
+}
+
 #endif // WITH_SQL
 
 } // qpdfview
