@@ -1,5 +1,6 @@
 /*
 
+Copyright 2014 S. Razi Alavizadeh
 Copyright 2013-2014 Adam Reichold
 
 This file is part of qpdfview.
@@ -30,13 +31,14 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #endif // WITH_SQL
 
+class QDateTime;
+
 #include "global.h"
 
 namespace qpdfview
 {
 
 class DocumentView;
-class BookmarkMenu;
 
 class Database : public QObject
 {
@@ -49,17 +51,18 @@ public:
     QStringList loadInstanceNames();
 
     void restoreTabs();
-    void saveTabs(const QList< const DocumentView* >& tabs);
+    void saveTabs(const QList< DocumentView* >& tabs);
+    void clearTabs();
 
     void restoreBookmarks();
-    void saveBookmarks(const QList< const BookmarkMenu* >& bookmarks);
+    void saveBookmarks();
+    void clearBookmarks();
 
     void restorePerFileSettings(DocumentView* tab);
     void savePerFileSettings(const DocumentView* tab);
 
 signals:
-    void tabRestored(const QString& absoluteFilePath, bool continuousMode, LayoutMode layoutMode, ScaleMode scaleMode, qreal scaleFactor, Rotation rotation, int currentPage);
-    void bookmarkRestored(const QString& absoluteFilePath, const JumpList& pages);
+    void tabRestored(const QString& absoluteFilePath, bool continuousMode, LayoutMode layoutMode, bool rightToLeftMode, ScaleMode scaleMode, qreal scaleFactor, Rotation rotation, int currentPage);
 
 private:
     Q_DISABLE_COPY(Database)
@@ -71,13 +74,18 @@ private:
 
 #ifdef WITH_SQL
 
-    bool prepareTabs_v2();
-    bool prepareBookmarks_v2();
-    bool preparePerFileSettings_v2();
+    bool prepareTabs_v3();
+    bool prepareBookmarks_v3();
+    bool preparePerFileSettings_v3();
 
-    void migrateTabs_v1_v2();
-    void migrateBookmarks_v1_v2();
-    void migratePerFileSettings_v1_v2();
+    void migrateTabs_v2_v3();
+    void migrateTabs_v1_v3();
+    void migrateBookmarks_v2_v3();
+    void migrateBookmarks_v1_v3();
+    void migratePerFileSettings_v2_v3();
+    void migratePerFileSettings_v1_v3();
+
+    void limitPerFileSettings();
 
     QSqlDatabase m_database;
 
