@@ -1,6 +1,7 @@
 /*
 
-Copyright 2013 Adam Reichold
+Copyright 2014 S. Razi Alavizadeh
+Copyright 2013-2014 Adam Reichold
 Copyright 2013 Alexander Volkov
 
 This file is part of qpdfview.
@@ -443,23 +444,31 @@ QList< Link* > DjVuPage::links() const
 
     miniexp_t pageAnnoExp;
 
-    while(true)
     {
-        pageAnnoExp = ddjvu_document_get_pageanno(m_parent->m_document, m_index);
+        QMutexLocker globalMutexLocker(m_parent->m_globalMutex);
 
-        if(pageAnnoExp == miniexp_dummy)
+        while(true)
         {
-            clearMessageQueue(m_parent->m_context, true);
-        }
-        else
-        {
-            break;
+            pageAnnoExp = ddjvu_document_get_pageanno(m_parent->m_document, m_index);
+
+            if(pageAnnoExp == miniexp_dummy)
+            {
+                clearMessageQueue(m_parent->m_context, true);
+            }
+            else
+            {
+                break;
+            }
         }
     }
 
     QList< Link* > links = loadLinks(pageAnnoExp, m_size, m_index, m_parent->m_indexByName);
 
-    ddjvu_miniexp_release(m_parent->m_document, pageAnnoExp);
+    {
+        QMutexLocker globalMutexLocker(m_parent->m_globalMutex);
+
+        ddjvu_miniexp_release(m_parent->m_document, pageAnnoExp);
+    }
 
     return links;
 }
@@ -470,23 +479,31 @@ QString DjVuPage::text(const QRectF& rect) const
 
     miniexp_t pageTextExp;
 
-    while(true)
     {
-        pageTextExp = ddjvu_document_get_pagetext(m_parent->m_document, m_index, "word");
+        QMutexLocker globalMutexLocker(m_parent->m_globalMutex);
 
-        if(pageTextExp == miniexp_dummy)
+        while(true)
         {
-            clearMessageQueue(m_parent->m_context, true);
-        }
-        else
-        {
-            break;
+            pageTextExp = ddjvu_document_get_pagetext(m_parent->m_document, m_index, "word");
+
+            if(pageTextExp == miniexp_dummy)
+            {
+                clearMessageQueue(m_parent->m_context, true);
+            }
+            else
+            {
+                break;
+            }
         }
     }
 
     const QString text = loadText(pageTextExp, QTransform::fromScale(m_resolution / 72.0, m_resolution / 72.0).mapRect(rect).toRect(), m_size.height());
 
-    ddjvu_miniexp_release(m_parent->m_document, pageTextExp);
+    {
+        QMutexLocker globalMutexLocker(m_parent->m_globalMutex);
+
+        ddjvu_miniexp_release(m_parent->m_document, pageTextExp);
+    }
 
     return text.trimmed();
 }
@@ -497,17 +514,21 @@ QList< QRectF > DjVuPage::search(const QString& text, bool matchCase) const
 
     miniexp_t pageTextExp;
 
-    while(true)
     {
-        pageTextExp = ddjvu_document_get_pagetext(m_parent->m_document, m_index, "word");
+        QMutexLocker globalMutexLocker(m_parent->m_globalMutex);
 
-        if(pageTextExp == miniexp_dummy)
+        while(true)
         {
-            clearMessageQueue(m_parent->m_context, true);
-        }
-        else
-        {
-            break;
+            pageTextExp = ddjvu_document_get_pagetext(m_parent->m_document, m_index, "word");
+
+            if(pageTextExp == miniexp_dummy)
+            {
+                clearMessageQueue(m_parent->m_context, true);
+            }
+            else
+            {
+                break;
+            }
         }
     }
 
@@ -519,6 +540,8 @@ QList< QRectF > DjVuPage::search(const QString& text, bool matchCase) const
     QRectF rect;
     int index = 0;
 
+    const QTransform transform = QTransform::fromScale(72.0 / m_resolution, 72.0 / m_resolution);
+
     while(!words.isEmpty())
     {
         miniexp_t textExp = words.takeFirst();
@@ -531,25 +554,21 @@ QList< QRectF > DjVuPage::search(const QString& text, bool matchCase) const
             {
                 const QString word = QString::fromUtf8(miniexp_to_str(miniexp_nth(5, textExp)));
 
-                if(text.indexOf(word, index, matchCase ? Qt::CaseSensitive : Qt::CaseInsensitive) == index)
+                index = word.indexOf(text, index, matchCase ? Qt::CaseSensitive : Qt::CaseInsensitive);
+
+                if(index != -1)
                 {
                     const int xmin = miniexp_to_int(miniexp_nth(1, textExp));
                     const int ymin = miniexp_to_int(miniexp_nth(2, textExp));
                     const int xmax = miniexp_to_int(miniexp_nth(3, textExp));
                     const int ymax = miniexp_to_int(miniexp_nth(4, textExp));
 
+                    index += text.length();
                     rect = rect.united(QRectF(xmin, m_size.height() - ymax, xmax - xmin, ymax - ymin));
 
-                    index += word.length();
-
-                    while(text.length() > index && text.at(index).isSpace())
+                    if(index == word.length() || !word.at(index).isLetter())
                     {
-                        ++index;
-                    }
-
-                    if(text.length() == index)
-                    {
-                        results.append(rect);
+                        results.append(transform.mapRect(rect));
 
                         rect = QRectF();
                         index = 0;
@@ -571,20 +590,18 @@ QList< QRectF > DjVuPage::search(const QString& text, bool matchCase) const
         }
     }
 
-    ddjvu_miniexp_release(m_parent->m_document, pageTextExp);
-
-    const QTransform transform = QTransform::fromScale(72.0 / m_resolution, 72.0 / m_resolution);
-
-    for(int index = 0; index < results.size(); ++index)
     {
-        results[index] = transform.mapRect(results[index]);
+        QMutexLocker globalMutexLocker(m_parent->m_globalMutex);
+
+        ddjvu_miniexp_release(m_parent->m_document, pageTextExp);
     }
 
     return results;
 }
 
-DjVuDocument::DjVuDocument(ddjvu_context_t* context, ddjvu_document_t* document) :
+DjVuDocument::DjVuDocument(QMutex* globalMutex, ddjvu_context_t* context, ddjvu_document_t* document) :
     m_mutex(),
+    m_globalMutex(globalMutex),
     m_context(context),
     m_document(document),
     m_format(0),
@@ -697,33 +714,34 @@ void DjVuDocument::loadOutline(QStandardItemModel* outlineModel) const
 
     miniexp_t outlineExp;
 
-    while(true)
     {
-        outlineExp = ddjvu_document_get_outline(m_document);
+        QMutexLocker globalMutexLocker(m_globalMutex);
 
-        if(outlineExp == miniexp_dummy)
+        while(true)
         {
-            clearMessageQueue(m_context, true);
-        }
-        else
-        {
-            break;
+            outlineExp = ddjvu_document_get_outline(m_document);
+
+            if(outlineExp == miniexp_dummy)
+            {
+                clearMessageQueue(m_context, true);
+            }
+            else
+            {
+                break;
+            }
         }
     }
 
-    if(miniexp_length(outlineExp) <= 1)
+    if(miniexp_length(outlineExp) > 1 && qstrncmp(miniexp_to_name(miniexp_nth(0, outlineExp)), "bookmarks", 9) == 0)
     {
-        return;
+        ::loadOutline(outlineExp, 1, outlineModel->invisibleRootItem(), m_indexByName);
     }
 
-    if(qstrncmp(miniexp_to_name(miniexp_nth(0, outlineExp)), "bookmarks", 9) != 0)
     {
-        return;
+        QMutexLocker globalMutexLocker(m_globalMutex);
+
+        ddjvu_miniexp_release(m_document, outlineExp);
     }
-
-    ::loadOutline(outlineExp, 1, outlineModel->invisibleRootItem(), m_indexByName);
-
-    ddjvu_miniexp_release(m_document, outlineExp);
 }
 
 void DjVuDocument::loadProperties(QStandardItemModel* propertiesModel) const
@@ -736,17 +754,21 @@ void DjVuDocument::loadProperties(QStandardItemModel* propertiesModel) const
 
     miniexp_t annoExp;
 
-    while(true)
     {
-        annoExp = ddjvu_document_get_anno(m_document, TRUE);
+        QMutexLocker globalMutexLocker(m_globalMutex);
 
-        if(annoExp == miniexp_dummy)
+        while(true)
         {
-            clearMessageQueue(m_context, true);
-        }
-        else
-        {
-            break;
+            annoExp = ddjvu_document_get_anno(m_document, TRUE);
+
+            if(annoExp == miniexp_dummy)
+            {
+                clearMessageQueue(m_context, true);
+            }
+            else
+            {
+                break;
+            }
         }
     }
 
@@ -781,12 +803,17 @@ void DjVuDocument::loadProperties(QStandardItemModel* propertiesModel) const
         }
     }
 
-    ddjvu_miniexp_release(m_document, annoExp);
+    {
+        QMutexLocker globalMutexLocker(m_globalMutex);
+
+        ddjvu_miniexp_release(m_document, annoExp);
+    }
 }
 
 } // Model
 
-DjVuPlugin::DjVuPlugin(QObject* parent) : QObject(parent)
+DjVuPlugin::DjVuPlugin(QObject* parent) : QObject(parent),
+    m_globalMutex()
 {
     setObjectName("DjVuPlugin");
 }
@@ -794,8 +821,22 @@ DjVuPlugin::DjVuPlugin(QObject* parent) : QObject(parent)
 Model::Document* DjVuPlugin::loadDocument(const QString& filePath) const
 {
     ddjvu_context_t* context = ddjvu_context_create("qpdfview");
+
+    if(context == 0)
+    {
+        return 0;
+    }
+
+#if DDJVUAPI_VERSION >= 19
+
+    ddjvu_document_t* document = ddjvu_document_create_by_filename_utf8(context, filePath.toUtf8(), FALSE);
+
+#else
+
     ddjvu_document_t* document = ddjvu_document_create_by_filename(context, QFile::encodeName(filePath), FALSE);
 
+#endif // DDJVUAPI_VERSION
+
     if(document == 0)
     {
         ddjvu_context_release(context);
@@ -813,7 +854,7 @@ Model::Document* DjVuPlugin::loadDocument(const QString& filePath) const
         return 0;
     }
 
-    return new Model::DjVuDocument(context, document);
+    return new Model::DjVuDocument(&m_globalMutex, context, document);
 }
 
 } // qpdfview
