@@ -1,6 +1,7 @@
 /*
 
-Copyright 2013 Adam Reichold
+Copyright 2014 S. Razi Alavizadeh
+Copyright 2013-2014 Adam Reichold
 
 This file is part of qpdfview.
 
@@ -49,6 +50,24 @@ DocumentLayout* DocumentLayout::fromLayoutMode(LayoutMode layoutMode)
     }
 }
 
+bool DocumentLayout::isCurrentPage(const QRectF& visibleRect, const QRectF& pageRect) const
+{
+    // Works with vertically scrolling layouts, i.e. all currently implemented layouts.
+    const qreal pageVisibleHeight = pageRect.intersected(visibleRect).height();
+    const qreal pageTopOffset = pageRect.top() - visibleRect.top();
+
+    if(visibleRect.height() > 2.0 * pageRect.height()) // Are more than two pages visible?
+    {
+        const qreal halfPageHeight = pageRect.height() / 2.0;
+
+        return pageVisibleHeight >= halfPageHeight && pageTopOffset < halfPageHeight && pageTopOffset >= -halfPageHeight;
+    }
+    else
+    {
+        return pageVisibleHeight >= visibleRect.height() / 2.0;
+    }
+}
+
 qreal DocumentLayout::visibleHeight(int viewportHeight) const
 {
     const qreal pageSpacing = s_settings->documentView().pageSpacing();
@@ -99,8 +118,8 @@ qreal SinglePageLayout::visibleWidth(int viewportWidth) const
     return viewportWidth - 6.0 - 2.0 * pageSpacing;
 }
 
-void SinglePageLayout::prepareLayout(const QVector<PageItem *>& pageItems, bool /* rightToLeft */,
-                                     QMap< qreal, int >& heightToIndex, qreal& left, qreal& right, qreal& height)
+void SinglePageLayout::prepareLayout(const QVector< PageItem* >& pageItems, bool /* rightToLeft */,
+                                     qreal& left, qreal& right, qreal& height)
 {
     const qreal pageSpacing = s_settings->documentView().pageSpacing();
     qreal pageHeight = 0.0;
@@ -112,8 +131,6 @@ void SinglePageLayout::prepareLayout(const QVector<PageItem *>& pageItems, bool
 
         page->setPos(-boundingRect.left() - 0.5 * boundingRect.width(), height - boundingRect.top());
 
-        heightToIndex.insert(-height + pageSpacing + 0.3 * pageHeight, index);
-
         pageHeight = boundingRect.height();
 
         left = qMin(left, -0.5f * boundingRect.width() - pageSpacing);
@@ -163,8 +180,8 @@ qreal TwoPagesLayout::visibleWidth(int viewportWidth) const
     return (viewportWidth - 6.0 - 3 * pageSpacing) / 2;
 }
 
-void TwoPagesLayout::prepareLayout(const QVector<PageItem *>& pageItems, bool rightToLeft,
-                                   QMap< qreal, int >& heightToIndex, qreal& left, qreal& right, qreal& height)
+void TwoPagesLayout::prepareLayout(const QVector< PageItem* >& pageItems, bool rightToLeft,
+                                   qreal& left, qreal& right, qreal& height)
 {
     const qreal pageSpacing = s_settings->documentView().pageSpacing();
     qreal pageHeight = 0.0;
@@ -181,8 +198,6 @@ void TwoPagesLayout::prepareLayout(const QVector<PageItem *>& pageItems, bool ri
         {
             page->setPos(rightToLeft ? rightPos : leftPos, height - boundingRect.top());
 
-            heightToIndex.insert(-height + pageSpacing + 0.3 * pageHeight, index);
-
             pageHeight = boundingRect.height();
 
             if(rightToLeft)
@@ -290,7 +305,7 @@ qreal MultiplePagesLayout::visibleWidth(int viewportWidth) const
 }
 
 void MultiplePagesLayout::prepareLayout(const QVector< PageItem* >& pageItems, bool rightToLeft,
-                                        QMap< qreal, int >& heightToIndex, qreal& left, qreal& right, qreal& height)
+                                        qreal& left, qreal& right, qreal& height)
 {
     const qreal pageSpacing = s_settings->documentView().pageSpacing();
     qreal pageHeight = 0.0;
@@ -316,11 +331,6 @@ void MultiplePagesLayout::prepareLayout(const QVector< PageItem* >& pageItems, b
             left += boundingRect.width() + pageSpacing;
         }
 
-        if(index == leftIndex(index))
-        {
-            heightToIndex.insert(-height + pageSpacing + 0.3 * pageHeight, index);
-        }
-
         if(index == rightIndex(index, pageItems.count()))
         {
             height += pageHeight + pageSpacing;
