@@ -1,8 +1,9 @@
 /*
 
+Copyright 2014 S. Razi Alavizadeh
+Copyright 2013 Thomas Etter
 Copyright 2012-2014 Adam Reichold
 Copyright 2014 Dorian Scholz
-Copyright 2013 Thomas Etter
 
 This file is part of qpdfview.
 
@@ -45,6 +46,7 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #ifdef WITH_CUPS
 
 #include <cups/cups.h>
+#include <cups/ppd.h>
 
 #endif // WITH_CUPS
 
@@ -59,11 +61,160 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include "pluginhandler.h"
 #include "shortcuthandler.h"
 #include "pageitem.h"
+#include "thumbnailitem.h"
 #include "presentationview.h"
 #include "searchtask.h"
 #include "miscellaneous.h"
 #include "documentlayout.h"
 
+namespace
+{
+
+using namespace qpdfview;
+
+// taken from http://rosettacode.org/wiki/Roman_numerals/Decode#C.2B.2B
+int romanToInt(const QString& text)
+{
+    if(text.size() == 1)
+    {
+        switch(text.at(0).toLower().toLatin1())
+        {
+        case 'i': return 1;
+        case 'v': return 5;
+        case 'x': return 10;
+        case 'l': return 50;
+        case 'c': return 100;
+        case 'd': return 500;
+        case 'm': return 1000;
+        }
+
+        return 0;
+    }
+
+    int result = 0;
+    int previous = 0, current = 0;
+
+    for(int i = text.size() - 1; i >= 0; --i)
+    {
+        current = romanToInt(text.at(i));
+
+        result += current < previous ? -current : current;
+
+        previous = current;
+    }
+
+    return result;
+}
+
+// taken from http://rosettacode.org/wiki/Roman_numerals/Encode#C.2B.2B
+QString intToRoman(int number)
+{
+    struct romandata_t
+    {
+        int value;
+        char const* numeral;
+    };
+
+    static const romandata_t romandata[] =
+    {
+        1000, "m",
+        900, "cm",
+        500, "d",
+        400, "cd",
+        100, "c",
+        90, "xc",
+        50, "l",
+        40, "xl",
+        10, "x",
+        9, "ix",
+        5, "v",
+        4, "iv",
+        1, "i",
+        0, NULL
+    };
+
+    if(number >= 4000)
+    {
+        return QLatin1String("?");
+    }
+
+    QString result;
+
+    for(const romandata_t* current = romandata; current->value > 0; ++current)
+    {
+        while(number >= current->value)
+        {
+            number -= current->value;
+            result += QLatin1String(current->numeral);
+        }
+    }
+
+    return result;
+}
+
+#ifdef WITH_CUPS
+
+struct RemovePpdFileDeleter
+{
+    static inline void cleanup(const char* ppdFileName) { if(ppdFileName != 0) { QFile::remove(ppdFileName); } }
+};
+
+struct ClosePpdFileDeleter
+{
+    static inline void cleanup(ppd_file_t* ppdFile) { if(ppdFile != 0) { ppdClose(ppdFile); } }
+};
+
+int addCMYKorRGBColorModel(cups_dest_t* dest, int num_options, cups_option_t** options)
+{
+    QScopedPointer< const char, RemovePpdFileDeleter > ppdFileName(cupsGetPPD(dest->name));
+
+    if(ppdFileName.isNull())
+    {
+        return num_options;
+    }
+
+    QScopedPointer< ppd_file_t, ClosePpdFileDeleter > ppdFile(ppdOpenFile(ppdFileName.data()));
+
+    if(ppdFile.isNull())
+    {
+        return num_options;
+    }
+
+    ppd_option_t* colorModel = ppdFindOption(ppdFile.data(), "ColorModel");
+
+    if(colorModel == 0)
+    {
+        return num_options;
+    }
+
+    for(int index = 0; index < colorModel->num_choices; ++index)
+    {
+        if(qstrncmp(colorModel->choices[index].choice, "CMYK", 4) == 0)
+        {
+            return cupsAddOption("ColorModel", "CMYK", num_options, options);
+        }
+    }
+
+    for(int index = 0; index < colorModel->num_choices; ++index)
+    {
+        if(qstrncmp(colorModel->choices[index].choice, "RGB", 3) == 0)
+        {
+            return cupsAddOption("ColorModel", "RGB", num_options, options);
+        }
+    }
+
+    return num_options;
+}
+
+#endif // WITH_CUPS
+
+bool modifiersUseMouseButton(Settings* settings, Qt::MouseButton mouseButton)
+{
+    return ((settings->documentView().zoomModifiers() | settings->documentView().rotateModifiers() | settings->documentView().scrollModifiers()) & mouseButton) != 0;
+}
+
+} // anonymous
+
 namespace qpdfview
 {
 
@@ -79,6 +230,7 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
     m_fileInfo(),
     m_wasModified(false),
     m_currentPage(-1),
+    m_firstPage(-1),
     m_past(),
     m_future(),
     m_layout(new SinglePageLayout),
@@ -91,7 +243,6 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
     m_rubberBandMode(ModifiersMode),
     m_pageItems(),
     m_thumbnailItems(),
-    m_heightToIndex(),
     m_highlight(0),
     m_thumbnailsOrientation(Qt::Vertical),
     m_thumbnailsScene(0),
@@ -115,9 +266,8 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
 
     setAcceptDrops(false);
     setDragMode(QGraphicsView::ScrollHandDrag);
-    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
 
-    connect(verticalScrollBar(), SIGNAL(valueChanged(int)), SLOT(on_verticalScrollBar_valueChanged(int)));
+    connect(verticalScrollBar(), SIGNAL(valueChanged(int)), SLOT(on_verticalScrollBar_valueChanged()));
 
     m_thumbnailsScene = new QGraphicsScene(this);
 
@@ -164,6 +314,7 @@ DocumentView::DocumentView(QWidget* parent) : QGraphicsView(parent),
     connect(this, SIGNAL(scaleModeChanged(ScaleMode)), m_prefetchTimer, SLOT(start()));
     connect(this, SIGNAL(scaleFactorChanged(qreal)), m_prefetchTimer, SLOT(start()));
     connect(this, SIGNAL(rotationChanged(Rotation)), m_prefetchTimer, SLOT(start()));
+    connect(this, SIGNAL(invertColorsChanged(bool)), m_prefetchTimer, SLOT(start()));
 
     connect(m_prefetchTimer, SIGNAL(timeout()), SLOT(on_prefetch_timeout()));
 
@@ -193,6 +344,107 @@ DocumentView::~DocumentView()
     delete m_document;
 }
 
+void DocumentView::setFirstPage(int firstPage)
+{
+    if(m_firstPage != firstPage)
+    {
+        m_firstPage = firstPage;
+
+        qDeleteAll(m_thumbnailItems);
+
+        prepareThumbnails();
+        prepareThumbnailsScene();
+
+        m_document->loadOutline(m_outlineModel);
+
+        if(m_outlineModel->rowCount() == 0)
+        {
+            loadFallbackOutline();
+        }
+
+        emit currentPageChanged(m_currentPage);
+    }
+}
+
+QString DocumentView::defaultPageLabelFromNumber(int number) const
+{
+    QLocale modifiedLocale = locale();
+
+    modifiedLocale.setNumberOptions(modifiedLocale.numberOptions() | QLocale::OmitGroupSeparator);
+
+    return modifiedLocale.toString(number);
+}
+
+QString DocumentView::pageLabelFromNumber(int number) const
+{
+    QString label;
+
+    if(hasFrontMatter())
+    {
+        if (number < m_firstPage)
+        {
+            label = number < 4000 ? intToRoman(number) : defaultPageLabelFromNumber(-number);
+        }
+        else
+        {
+            label = defaultPageLabelFromNumber(number - m_firstPage + 1);
+        }
+    }
+    else if(number >= 1 && number <= m_pages.count())
+    {
+        const QString& pageLabel = m_pages.at(number - 1)->label();
+
+        if(number != pageLabel.toInt())
+        {
+            label = pageLabel;
+        }
+    }
+
+    if(label.isEmpty())
+    {
+        label = defaultPageLabelFromNumber(number);
+    }
+
+    return label;
+}
+
+int DocumentView::pageNumberFromLabel(const QString& label) const
+{
+    if(hasFrontMatter())
+    {
+        bool ok = false;
+        int value = locale().toInt(label, &ok);
+
+        if(ok)
+        {
+            if(value < 0)
+            {
+                value = -value; // front matter
+            }
+            else
+            {
+                value = value + m_firstPage - 1; // body matter
+            }
+        }
+        else
+        {
+            value = romanToInt(label);
+        }
+
+        return value;
+    }
+
+    for(int index = 0; index < m_pages.count(); ++index)
+    {
+        if(m_pages.at(index)->label() == label)
+        {
+            return index + 1;
+        }
+    }
+
+    return locale().toInt(label);
+}
+
 QStringList DocumentView::openFilter()
 {
     QStringList openFilter;
@@ -245,6 +497,8 @@ void DocumentView::setContinuousMode(bool continuousMode)
         qreal left = 0.0, top = 0.0;
         saveLeftAndTop(left, top);
 
+        adjustScrollBarPolicy();
+
         prepareView(left, top);
 
         emit continuousModeChanged(m_continuousMode);
@@ -304,6 +558,8 @@ void DocumentView::setScaleMode(ScaleMode scaleMode)
         qreal left = 0.0, top = 0.0;
         saveLeftAndTop(left, top);
 
+        adjustScrollBarPolicy();
+
         prepareScene();
         prepareView(left, top);
 
@@ -315,7 +571,9 @@ void DocumentView::setScaleMode(ScaleMode scaleMode)
 
 void DocumentView::setScaleFactor(qreal scaleFactor)
 {
-    if(!qFuzzyCompare(m_scaleFactor, scaleFactor) && scaleFactor >= Defaults::DocumentView::minimumScaleFactor() && scaleFactor <= Defaults::DocumentView::maximumScaleFactor())
+    if(!qFuzzyCompare(m_scaleFactor, scaleFactor)
+            && scaleFactor >= s_settings->documentView().minimumScaleFactor()
+            && scaleFactor <= s_settings->documentView().maximumScaleFactor())
     {
         m_scaleFactor = scaleFactor;
 
@@ -470,6 +728,8 @@ bool DocumentView::open(const QString& filePath)
 
         loadDocumentDefaults();
 
+        adjustScrollBarPolicy();
+
         prepareScene();
         prepareView();
 
@@ -481,6 +741,10 @@ bool DocumentView::open(const QString& filePath)
         emit currentPageChanged(m_currentPage);
 
         emit canJumpChanged(false, false);
+
+        emit continuousModeChanged(m_continuousMode);
+        emit layoutModeChanged(m_layout->layoutMode());
+        emit rightToLeftModeChanged(m_rightToLeftMode);
     }
 
     return document != 0;
@@ -568,7 +832,7 @@ bool DocumentView::save(const QString& filePath, bool withChanges)
 }
 
 bool DocumentView::print(QPrinter* printer, const PrintOptions& printOptions)
-{    
+{
     const int fromPage = printer->fromPage() != 0 ? printer->fromPage() : 1;
     const int toPage = printer->toPage() != 0 ? printer->toPage() : m_pages.count();
 
@@ -786,12 +1050,15 @@ void DocumentView::zoomIn()
 {
     if(scaleMode() != ScaleFactorMode)
     {
-        setScaleFactor(qMin(m_pageItems.at(m_currentPage - 1)->scaleFactor() + Defaults::DocumentView::zoomBy(), Defaults::DocumentView::maximumScaleFactor()));
+        setScaleFactor(qMin(m_pageItems.at(m_currentPage - 1)->scaleFactor() * s_settings->documentView().zoomFactor(),
+                            s_settings->documentView().maximumScaleFactor()));
+
         setScaleMode(ScaleFactorMode);
     }
     else
     {
-        setScaleFactor(qMin(m_scaleFactor + Defaults::DocumentView::zoomBy(), Defaults::DocumentView::maximumScaleFactor()));
+        setScaleFactor(qMin(m_scaleFactor * s_settings->documentView().zoomFactor(),
+                            s_settings->documentView().maximumScaleFactor()));
     }
 }
 
@@ -799,12 +1066,15 @@ void DocumentView::zoomOut()
 {
     if(scaleMode() != ScaleFactorMode)
     {
-        setScaleFactor(qMax(m_pageItems.at(m_currentPage - 1)->scaleFactor() - Defaults::DocumentView::zoomBy(), Defaults::DocumentView::minimumScaleFactor()));
+        setScaleFactor(qMax(m_pageItems.at(m_currentPage - 1)->scaleFactor() / s_settings->documentView().zoomFactor(),
+                            s_settings->documentView().minimumScaleFactor()));
+
         setScaleMode(ScaleFactorMode);
     }
     else
     {
-        setScaleFactor(qMax(m_scaleFactor - Defaults::DocumentView::zoomBy(), Defaults::DocumentView::minimumScaleFactor()));
+        setScaleFactor(qMax(m_scaleFactor / s_settings->documentView().zoomFactor(),
+                            s_settings->documentView().minimumScaleFactor()));
     }
 }
 
@@ -880,41 +1150,44 @@ void DocumentView::startPresentation()
     }
 }
 
-void DocumentView::on_verticalScrollBar_valueChanged(int value)
+void DocumentView::on_verticalScrollBar_valueChanged()
 {
     if(!m_continuousMode)
     {
         return;
     }
 
+    int currentPage = -1;
     const QRectF visibleRect = mapToScene(viewport()->rect()).boundingRect();
 
     foreach(PageItem* page, m_pageItems)
     {
-        if(!page->boundingRect().translated(page->pos()).intersects(visibleRect))
+        const int pageNumber = page->index() + 1;
+        const QRectF pageRect = page->boundingRect().translated(page->pos());
+
+        if(!pageRect.intersects(visibleRect))
         {
             page->cancelRender();
         }
+        else if(currentPage == -1 &&
+                 m_layout->currentPage(pageNumber) == pageNumber &&
+                 m_layout->isCurrentPage(visibleRect, pageRect))
+        {
+            currentPage = pageNumber;
+        }
     }
 
-    const QMap< qreal, int >::iterator lowerBound = m_heightToIndex.lowerBound(-value);
-
-    if(lowerBound != m_heightToIndex.end())
+    if(currentPage != -1 && m_currentPage != currentPage)
     {
-        const int page = lowerBound.value() + 1;
+        m_currentPage = currentPage;
 
-        if(m_currentPage != page)
+        emit currentPageChanged(m_currentPage);
+
+        if(s_settings->documentView().highlightCurrentThumbnail())
         {
-            m_currentPage = page;
-
-            emit currentPageChanged(m_currentPage);
-
-            if(s_settings->documentView().highlightCurrentThumbnail())
+            for(int index = 0; index < m_thumbnailItems.count(); ++index)
             {
-                for(int index = 0; index < m_thumbnailItems.count(); ++index)
-                {
-                    m_thumbnailItems.at(index)->setCurrent(index == m_currentPage - 1);
-                }
+                m_thumbnailItems.at(index)->setHighlighted(index == m_currentPage - 1);
             }
         }
     }
@@ -938,9 +1211,27 @@ void DocumentView::on_prefetch_timeout()
 {
     const QPair< int, int > prefetchRange = m_layout->prefetchRange(m_currentPage, m_pages.count());
 
-    for(int index = prefetchRange.first - 1; index <= prefetchRange.second - 1; ++index)
+    const int maxCost = prefetchRange.second - prefetchRange.first + 1;
+    int cost = 0;
+
+    for(int index = m_currentPage - 1; index <= prefetchRange.second - 1; ++index)
     {
-        m_pageItems.at(index)->startRender(true);
+        cost += m_pageItems.at(index)->startRender(true);
+
+        if(cost >= maxCost)
+        {
+            return;
+        }
+    }
+
+    for(int index = m_currentPage - 1; index >= prefetchRange.first - 1; --index)
+    {
+        cost += m_pageItems.at(index)->startRender(true);
+
+        if(cost >= maxCost)
+        {
+            return;
+        }
     }
 }
 
@@ -974,10 +1265,26 @@ void DocumentView::on_searchTask_resultsReady(int index, QList< QRectF > results
 
     if(m_results.contains(index) && m_currentResult == m_results.end())
     {
+        setFocus();
+
         findNext();
     }
 }
 
+void DocumentView::on_pages_cropRectChanged()
+{
+    qreal left = 0.0, top = 0.0;
+    saveLeftAndTop(left, top);
+
+    prepareScene();
+    prepareView(left, top);
+}
+
+void DocumentView::on_thumbnails_cropRectChanged()
+{
+    prepareThumbnailsScene();
+}
+
 void DocumentView::on_pages_linkClicked(bool newTab, int page, qreal left, qreal top)
 {
     page = qMax(page, 1);
@@ -1077,16 +1384,23 @@ void DocumentView::on_pages_editSourceRequested(int page, const QPointF& pos)
 
 void DocumentView::on_pages_zoomToSelectionRequested(int page, const QRectF& rect)
 {
+    if(rect.isEmpty())
+    {
+        return;
+    }
+
     const qreal visibleWidth = m_layout->visibleWidth(viewport()->width());
     const qreal visibleHeight = m_layout->visibleHeight(viewport()->height());
 
-    const qreal unscaledWidth = m_pageItems.at(page - 1)->unscaledWidth();
-    const qreal unscaledHeight = m_pageItems.at(page - 1)->unscaledHeight();
+    const qreal displayedWidth = m_pageItems.at(page - 1)->displayedWidth();
+    const qreal displayedHeight = m_pageItems.at(page - 1)->displayedHeight();
 
-    setScaleFactor(qMin(qMin(visibleWidth / unscaledWidth / rect.width(),
-                             visibleHeight / unscaledHeight / rect.height()),
+    setScaleFactor(qMin(qMin(visibleWidth / displayedWidth / rect.width(),
+                             visibleHeight / displayedHeight / rect.height()),
                         Defaults::DocumentView::maximumScaleFactor()));
 
+    setScaleMode(ScaleFactorMode);
+
     jumpToPage(page, false, rect.left(), rect.top());
 }
 
@@ -1209,7 +1523,11 @@ void DocumentView::keyPressEvent(QKeyEvent* event)
 
 void DocumentView::wheelEvent(QWheelEvent* event)
 {
-    if(event->modifiers() == s_settings->documentView().zoomModifiers())
+    const Qt::KeyboardModifiers zoomModifiers = s_settings->documentView().zoomModifiers();
+    const Qt::KeyboardModifiers rotateModifiers = s_settings->documentView().rotateModifiers();
+    const Qt::KeyboardModifiers scrollModifiers = s_settings->documentView().scrollModifiers();
+
+    if(event->modifiers() == zoomModifiers || event->buttons() == zoomModifiers)
     {
         if(event->delta() > 0)
         {
@@ -1223,7 +1541,7 @@ void DocumentView::wheelEvent(QWheelEvent* event)
         event->accept();
         return;
     }
-    else if(event->modifiers() == s_settings->documentView().rotateModifiers())
+    else if(event->modifiers() == rotateModifiers || event->buttons() == rotateModifiers)
     {
         if(event->delta() > 0)
         {
@@ -1237,7 +1555,7 @@ void DocumentView::wheelEvent(QWheelEvent* event)
         event->accept();
         return;
     }
-    else if(event->modifiers() == s_settings->documentView().scrollModifiers())
+    else if(event->modifiers() == scrollModifiers || event->buttons() == scrollModifiers)
     {
         QWheelEvent wheelEvent(event->pos(), event->delta(), event->buttons(), Qt::AltModifier, Qt::Horizontal);
         QGraphicsView::wheelEvent(&wheelEvent);
@@ -1275,6 +1593,12 @@ void DocumentView::wheelEvent(QWheelEvent* event)
 
 void DocumentView::contextMenuEvent(QContextMenuEvent* event)
 {
+    if(event->reason() == QContextMenuEvent::Mouse && modifiersUseMouseButton(s_settings, Qt::RightButton))
+    {
+        event->accept();
+        return;
+    }
+
     event->setAccepted(false);
 
     QGraphicsView::contextMenuEvent(event);
@@ -1351,6 +1675,7 @@ bool DocumentView::printUsingCUPS(QPrinter* printer, const PrintOptions& printOp
         switch(printer->colorMode())
         {
         case QPrinter::Color:
+            num_options = addCMYKorRGBColorModel(dest, num_options, &options);
             break;
         case QPrinter::GrayScale:
             num_options = cupsAddOption("ColorModel", "Gray", num_options, &options);
@@ -1374,6 +1699,8 @@ bool DocumentView::printUsingCUPS(QPrinter* printer, const PrintOptions& printOp
 
         int numberUp = 1;
 
+#if QT_VERSION < QT_VERSION_CHECK(5,2,0)
+
         switch(printOptions.numberUp)
         {
         case PrintOptions::SinglePage:
@@ -1402,9 +1729,6 @@ bool DocumentView::printUsingCUPS(QPrinter* printer, const PrintOptions& printOp
             break;
         }
 
-        fromPage = (fromPage - 1) / numberUp + 1;
-        toPage = (toPage - 1) / numberUp + 1;
-
         switch(printOptions.numberUpLayout)
         {
         case PrintOptions::BottomTopLeftRight:
@@ -1433,15 +1757,6 @@ bool DocumentView::printUsingCUPS(QPrinter* printer, const PrintOptions& printOp
             break;
         }
 
-        if(printOptions.pageRanges.isEmpty())
-        {
-            num_options = cupsAddOption("page-ranges", QString("%1-%2").arg(fromPage).arg(toPage).toLocal8Bit(), num_options, &options);
-        }
-        else
-        {
-            num_options = cupsAddOption("page-ranges", printOptions.pageRanges.toLocal8Bit(), num_options, &options);
-        }
-
         switch(printOptions.pageSet)
         {
         case PrintOptions::AllPages:
@@ -1454,6 +1769,29 @@ bool DocumentView::printUsingCUPS(QPrinter* printer, const PrintOptions& printOp
             break;
         }
 
+#else // QT_VERSION
+
+        {
+            bool ok = false;
+            int value = QString::fromLocal8Bit(cupsGetOption("number-up", num_options, options)).toInt(&ok);
+
+            numberUp = ok ? value : 1;
+        }
+
+#endif // QT_VERSION
+
+        fromPage = (fromPage - 1) / numberUp + 1;
+        toPage = (toPage - 1) / numberUp + 1;
+
+        if(printOptions.pageRanges.isEmpty())
+        {
+            num_options = cupsAddOption("page-ranges", QString("%1-%2").arg(fromPage).arg(toPage).toLocal8Bit(), num_options, &options);
+        }
+        else
+        {
+            num_options = cupsAddOption("page-ranges", printOptions.pageRanges.toLocal8Bit(), num_options, &options);
+        }
+
         QTemporaryFile temporaryFile;
 
         if(temporaryFile.open())
@@ -1550,9 +1888,6 @@ void DocumentView::saveLeftAndTop(qreal& left, qreal& top) const
 
     left = (topLeft.x() - boundingRect.x()) / boundingRect.width();
     top = (topLeft.y() - boundingRect.y()) / boundingRect.height();
-
-    left = left >= 0.0 ? left : 0.0;
-    top = top >= 0.0 ? top : 0.0;
 }
 
 bool DocumentView::checkDocument(const QString& filePath, Model::Document* document)
@@ -1581,7 +1916,7 @@ void DocumentView::loadFallbackOutline()
 {
     for(int page = 1; page <= m_pages.count(); ++page)
     {
-        QStandardItem* item = new QStandardItem(tr("Page %1").arg(page));
+        QStandardItem* item = new QStandardItem(tr("Page %1").arg(pageLabelFromNumber(page)));
         item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
 
         item->setData(page, Qt::UserRole + 1);
@@ -1620,6 +1955,26 @@ void DocumentView::loadDocumentDefaults()
     }
 }
 
+void DocumentView::adjustScrollBarPolicy()
+{
+    switch(m_scaleMode)
+    {
+    default:
+    case ScaleFactorMode:
+        setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
+        setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
+        break;
+    case FitToPageWidthMode:
+        setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+        setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
+        break;
+    case FitToPageSizeMode:
+        setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+        setVerticalScrollBarPolicy(m_continuousMode ? Qt::ScrollBarAsNeeded : Qt::ScrollBarAlwaysOff);
+        break;
+    }
+}
+
 void DocumentView::prepareDocument(Model::Document* document)
 {
     m_prefetchTimer->blockSignals(true);
@@ -1691,6 +2046,8 @@ void DocumentView::preparePages()
         scene()->addItem(page);
         m_pageItems.append(page);
 
+        connect(page, SIGNAL(cropRectChanged()), SLOT(on_pages_cropRectChanged()));
+
         connect(page, SIGNAL(linkClicked(bool,int,qreal,qreal)), SLOT(on_pages_linkClicked(bool,int,qreal,qreal)));
         connect(page, SIGNAL(linkClicked(QString)), SLOT(on_pages_linkClicked(QString)));
         connect(page, SIGNAL(linkClicked(QString,int)), SLOT(on_pages_linkClicked(QString,int)));
@@ -1711,13 +2068,15 @@ void DocumentView::prepareThumbnails()
 
     for(int index = 0; index < m_pages.count(); ++index)
     {
-        ThumbnailItem* page = new ThumbnailItem(m_pages.at(index), index);
+        ThumbnailItem* page = new ThumbnailItem(m_pages.at(index), pageLabelFromNumber(index + 1), index);
 
         page->setInvertColors(m_invertColors);
 
         m_thumbnailsScene->addItem(page);
         m_thumbnailItems.append(page);
 
+        connect(page, SIGNAL(cropRectChanged()), SLOT(on_thumbnails_cropRectChanged()));
+
         connect(page, SIGNAL(linkClicked(bool,int,qreal,qreal)), SLOT(on_pages_linkClicked(bool,int,qreal,qreal)));
     }
 }
@@ -1763,8 +2122,8 @@ void DocumentView::prepareScene()
 
         page->setRotation(m_rotation);
 
-        const qreal unscaledWidth = page->unscaledWidth();
-        const qreal unscaledHeight = page->unscaledHeight();
+        const qreal displayedWidth = page->displayedWidth();
+        const qreal displayedHeight = page->displayedHeight();
 
         switch(m_scaleMode)
         {
@@ -1773,24 +2132,22 @@ void DocumentView::prepareScene()
             page->setScaleFactor(m_scaleFactor);
             break;
         case FitToPageWidthMode:
-            page->setScaleFactor(visibleWidth / unscaledWidth);
+            page->setScaleFactor(visibleWidth / displayedWidth);
             break;
         case FitToPageSizeMode:
-            page->setScaleFactor(qMin(visibleWidth / unscaledWidth, visibleHeight / unscaledHeight));
+            page->setScaleFactor(qMin(visibleWidth / displayedWidth, visibleHeight / displayedHeight));
             break;
         }
     }
 
     // prepare layout
 
-    m_heightToIndex.clear();
-
     qreal left = 0.0;
     qreal right = 0.0;
     qreal height = s_settings->documentView().pageSpacing();
 
     m_layout->prepareLayout(m_pageItems, m_rightToLeftMode,
-                            m_heightToIndex, left, right, height);
+                            left, right, height);
 
     scene()->setSceneRect(left, 0.0, right - left, height);
 }
@@ -1849,7 +2206,7 @@ void DocumentView::prepareView(qreal changeLeft, qreal changeTop, int visiblePag
             page->stackBefore(m_highlight);
         }
 
-        m_thumbnailItems.at(index)->setCurrent(highlightCurrentThumbnail && (index == m_currentPage - 1));
+        m_thumbnailItems.at(index)->setHighlighted(highlightCurrentThumbnail && (index == m_currentPage - 1));
     }
 
     setSceneRect(left, top, width, height);
@@ -1896,7 +2253,8 @@ void DocumentView::prepareThumbnailsScene()
 
         page->setResolution(logicalDpiX(), logicalDpiY());
 
-        page->setScaleFactor(qMin(s_settings->documentView().thumbnailSize() / page->unscaledWidth(), s_settings->documentView().thumbnailSize() / page->unscaledHeight()));
+        page->setScaleFactor(qMin(s_settings->documentView().thumbnailSize() / page->displayedWidth(),
+                                  s_settings->documentView().thumbnailSize() / page->displayedHeight()));
 
         // prepare layout
 
@@ -1937,9 +2295,9 @@ void DocumentView::prepareHighlight(int index, const QRectF& rect)
 
     m_highlight->setVisible(true);
 
-    disconnect(verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(on_verticalScrollBar_valueChanged(int)));
+    disconnect(verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(on_verticalScrollBar_valueChanged()));
     centerOn(m_highlight);
-    connect(verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(on_verticalScrollBar_valueChanged(int)));
+    connect(verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(on_verticalScrollBar_valueChanged()));
 
     viewport()->update();
 }
