@@ -1,5 +1,6 @@
 /*
 
+Copyright 2014 S. Razi Alavizadeh
 Copyright 2012-2014 Adam Reichold
 Copyright 2014 Dorian Scholz
 
@@ -51,7 +52,7 @@ class ThumbnailItem;
 class SearchTask;
 class PresentationView;
 class ShortcutHandler;
-class DocumentLayout;
+struct DocumentLayout;
 
 class DocumentView : public QGraphicsView
 {
@@ -67,6 +68,15 @@ public:
     inline int numberOfPages() const { return m_pages.count(); }
     inline int currentPage() const { return m_currentPage; }
 
+    inline bool hasFrontMatter() const { return m_firstPage > 1; }
+
+    inline int firstPage() const { return m_firstPage; }
+    void setFirstPage(int firstPage);
+
+    QString defaultPageLabelFromNumber(int number) const;
+    QString pageLabelFromNumber(int number) const;
+    int pageNumberFromLabel(const QString& label) const;
+
     static QStringList openFilter();
     QStringList saveFilter() const;
 
@@ -178,7 +188,7 @@ public slots:
     void startPresentation();
 
 protected slots:
-    void on_verticalScrollBar_valueChanged(int value);
+    void on_verticalScrollBar_valueChanged();
 
     void on_autoRefresh_timeout();
     void on_prefetch_timeout();
@@ -187,6 +197,9 @@ protected slots:
 
     void on_searchTask_resultsReady(int index, QList< QRectF > results);
 
+    void on_pages_cropRectChanged();
+    void on_thumbnails_cropRectChanged();
+
     void on_pages_linkClicked(bool newTab, int page, qreal left, qreal top);
     void on_pages_linkClicked(const QString& url);
     void on_pages_linkClicked(const QString& fileName, int page);
@@ -224,6 +237,7 @@ private:
     bool m_wasModified;
 
     int m_currentPage;
+    int m_firstPage;
 
 #ifdef WITH_CUPS
 
@@ -263,8 +277,6 @@ private:
     QVector< PageItem* > m_pageItems;
     QVector< ThumbnailItem* > m_thumbnailItems;
 
-    QMap< qreal, int > m_heightToIndex;
-
     QGraphicsRectItem* m_highlight;
 
     Qt::Orientation m_thumbnailsOrientation;
@@ -278,6 +290,8 @@ private:
     void loadFallbackOutline();
     void loadDocumentDefaults();
 
+    void adjustScrollBarPolicy();
+
     void prepareDocument(Model::Document* document);
     void preparePages();
     void prepareThumbnails();
