@@ -36,7 +36,8 @@ class FontsDialog : public QDialog
     Q_OBJECT
 
 public:
-    explicit FontsDialog(QStandardItemModel* model, QWidget* parent = 0);
+    FontsDialog(QStandardItemModel* model, QWidget* parent = 0);
+    ~FontsDialog();
 
 private:
     Q_DISABLE_COPY(FontsDialog)
