@@ -1,5 +1,6 @@
 /*
 
+Copyright 2014 S. Razi Alavizadeh
 Copyright 2012-2014 Adam Reichold
 Copyright 2012 Michał Trybus
 Copyright 2012 Alexander Volkov
@@ -26,6 +27,8 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <QMainWindow>
 
+#include <QPointer>
+
 #ifdef WITH_DBUS
 
 #include <QDBusAbstractAdaptor>
@@ -33,6 +36,7 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #endif // WITH_DBUS
 
 class QCheckBox;
+class QDateTime;
 class QGraphicsView;
 class QFileInfo;
 class QModelIndex;
@@ -50,16 +54,19 @@ class DocumentView;
 class TabWidget;
 class TreeView;
 class ComboBox;
-class SpinBox;
+class MappingSpinBox;
 class SearchLineEdit;
 class RecentlyUsedMenu;
 class RecentlyClosedMenu;
-class BookmarkMenu;
+class BookmarkModel;
 class Database;
+class HelpDialog;
 
 class MainWindow : public QMainWindow
 {
     Q_OBJECT
+
+    friend class MainWindowAdaptor;
     
 public:
     explicit MainWindow(QWidget* parent = 0);
@@ -92,6 +99,7 @@ protected slots:
     void on_currentTab_rightToLeftModeChanged(bool rightToLeftMode);
     void on_currentTab_scaleModeChanged(ScaleMode scaleMode);
     void on_currentTab_scaleFactorChanged(qreal scaleFactor);
+    void on_currentTab_rotationChanged(Rotation rotation);
 
     void on_currentTab_linkClicked(int page);
     void on_currentTab_linkClicked(const QString& filePath, int page);
@@ -127,6 +135,8 @@ protected slots:
     void on_firstPage_triggered();
     void on_lastPage_triggered();
 
+    void on_setFirstPage_triggered();
+
     void on_jumpToPage_triggered();
 
     void on_jumpBackward_triggered();
@@ -183,26 +193,41 @@ protected slots:
     void on_removeBookmark_triggered();
     void on_removeAllBookmarks_triggered();
 
+    void on_bookmarksMenu_aboutToShow();
+
     void on_bookmark_openTriggered(const QString& absoluteFilePath);
     void on_bookmark_openInNewTabTriggered(const QString& absoluteFilePath);
     void on_bookmark_jumpToPageTriggered(const QString& absoluteFilePath, int page);
+    void on_bookmark_removeBookmarkTriggered(const QString& absoluteFilePath);
 
     void on_contents_triggered();
     void on_about_triggered();
 
+    QString currentPage_textFromValue(int value, bool* ok) const;
+    int currentPage_valueFromText(QString text, bool* ok) const;
+
     void on_focusCurrentPage_activated();
     void on_focusScaleFactor_activated();
 
+    void on_toggleToolBars_triggered(bool checked);
+    void on_toggleMenuBar_triggered(bool checked);
+
     void on_searchInitiated(const QString& text, bool allTabs);
     void on_highlightAll_clicked(bool checked);
 
+    void on_outline_sectionCountChanged();
     void on_outline_clicked(const QModelIndex& index);
 
+    void on_properties_sectionCountChanged();
+
     void on_thumbnails_dockLocationChanged(Qt::DockWidgetArea area);
     void on_thumbnails_verticalScrollBar_valueChanged(int value);
 
-    void on_database_tabRestored(const QString& absoluteFilePath, bool continuousMode, LayoutMode layoutMode, ScaleMode scaleMode, qreal scaleFactor, Rotation rotation, int currentPage);
-    void on_database_bookmarkRestored(const QString& absoluteFilePath, const JumpList& pages);
+    void on_bookmarks_sectionCountChanged();
+    void on_bookmarks_clicked(const QModelIndex& index);
+    void on_bookmarks_contextMenuRequested(const QPoint& pos);
+
+    void on_database_tabRestored(const QString& absoluteFilePath, bool continuousMode, LayoutMode layoutMode, bool rightToLeftMode, ScaleMode scaleMode, qreal scaleFactor, Rotation rotation, int currentPage);
 
     void on_saveDatabase_timeout();
 
@@ -234,16 +259,20 @@ private:
     bool saveModifications(DocumentView* tab);
 
     void setWindowTitleForCurrentTab();
-    BookmarkMenu* bookmarkForCurrentTab() const;
+    void setCurrentPageSuffixForCurrentTab();
 
-    void prepareDatabase();
-
-    void saveTabs() const;
-    void saveBookmarks() const;
+    BookmarkModel* bookmarkModelForCurrentTab(bool create = false);
 
     QTimer* m_saveDatabaseTimer;
 
-    SpinBox* m_currentPageSpinBox;
+    void prepareDatabase();
+
+    void scheduleSaveDatabase();
+    void scheduleSaveTabs();
+    void scheduleSaveBookmarks();
+    void scheduleSavePerFileSettings();
+
+    MappingSpinBox* m_currentPageSpinBox;
     QWidgetAction* m_currentPageAction;
 
     ComboBox* m_scaleFactorComboBox;
@@ -269,6 +298,8 @@ private:
     QAction* m_firstPageAction;
     QAction* m_lastPageAction;
 
+    QAction* m_setFirstPageAction;
+
     QAction* m_jumpToPageAction;
 
     QAction* m_jumpBackwardAction;
@@ -327,8 +358,8 @@ private:
     QAction* m_contentsAction;
     QAction* m_aboutAction;
 
-    QAction* createAction(const QString& text, const QString& objectName, const QIcon& icon, const QKeySequence& shortcut, const char* member, bool checkable = false);
-    QAction* createAction(const QString& text, const QString& objectName, const QString& iconName, const QKeySequence& shortcut, const char* member, bool checkable = false);
+    QAction* createAction(const QString& text, const QString& objectName, const QIcon& icon, const QKeySequence& shortcut, const char* member, bool checkable = false, bool checked = false);
+    QAction* createAction(const QString& text, const QString& objectName, const QString& iconName, const QKeySequence& shortcut, const char* member, bool checkable = false, bool checked = false);
 
     void createActions();
 
@@ -352,6 +383,9 @@ private:
     QDockWidget* m_thumbnailsDock;
     QGraphicsView* m_thumbnailsView;
 
+    QDockWidget* m_bookmarksDock;
+    QTableView* m_bookmarksView;
+
     QDockWidget* m_searchDock;
     QWidget* m_searchWidget;
 
@@ -368,8 +402,21 @@ private:
     QMenu* m_bookmarksMenu;
     QMenu* m_helpMenu;
 
+    bool m_bookmarksMenuIsDirty;
+
     void createMenus();
 
+    int m_tabBarHadPolicy;
+
+    bool m_fileToolBarWasVisible;
+    bool m_editToolBarWasVisible;
+    bool m_viewToolBarWasVisible;
+
+    QAction* m_toggleToolBarsAction;
+    QAction* m_toggleMenuBarAction;
+
+    QPointer< HelpDialog > m_helpDialog;
+
 };
 
 #ifdef WITH_DBUS
@@ -383,6 +430,8 @@ public:
     explicit MainWindowAdaptor(MainWindow* mainWindow);
 
 public slots:
+    Q_NOREPLY void raiseAndActivate();
+
     bool open(const QString& absoluteFilePath, int page = -1, const QRectF& highlight = QRectF(), bool quiet = false);
     bool openInNewTab(const QString& absoluteFilePath, int page = -1, const QRectF& highlight = QRectF(), bool quiet = false);
 
@@ -390,7 +439,37 @@ public slots:
 
     Q_NOREPLY void startSearch(const QString& text);
 
-    Q_NOREPLY void raiseAndActivate();
+
+    Q_NOREPLY void previousPage();
+    Q_NOREPLY void nextPage();
+    Q_NOREPLY void firstPage();
+    Q_NOREPLY void lastPage();
+
+    Q_NOREPLY void previousBookmark();
+    Q_NOREPLY void nextBookmark();
+
+    bool jumpToBookmark(const QString& label);
+
+
+    Q_NOREPLY void continuousModeAction(bool checked);
+    Q_NOREPLY void twoPagesModeAction(bool checked);
+    Q_NOREPLY void twoPagesWithCoverPageModeAction(bool checked);
+    Q_NOREPLY void multiplePagesModeAction(bool checked);
+
+    Q_NOREPLY void fitToPageWidthModeAction(bool checked);
+    Q_NOREPLY void fitToPageSizeModeAction(bool checked);
+
+    Q_NOREPLY void invertColorsAction(bool checked);
+
+    Q_NOREPLY void fullscreenAction(bool checked);
+    Q_NOREPLY void presentationAction();
+
+
+    Q_NOREPLY void closeTab();
+    Q_NOREPLY void closeAllTabs();
+    Q_NOREPLY void closeAllTabsButCurrentTab();
+
+    bool closeTab(const QString& absoluteFilePath);
 
 private:
     MainWindow* mainWindow() const;
