@@ -1,5 +1,6 @@
 /*
 
+Copyright 2014 S. Razi Alavizadeh
 Copyright 2012-2014 Adam Reichold
 Copyright 2014 Dorian Scholz
 
@@ -23,9 +24,33 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include "miscellaneous.h"
 
 #include <QApplication>
+#include <QDebug>
+#include <QDialog>
+#include <QDialogButtonBox>
+#include <QLabel>
 #include <QMenu>
 #include <QMouseEvent>
 #include <QTimer>
+#include <QVBoxLayout>
+
+namespace
+{
+
+using namespace qpdfview;
+
+QMetaMethod slotToMethod(const QMetaObject* metaObject, const char* slot)
+{
+    const int index = *slot == '1' ? metaObject->indexOfSlot(slot + 1) : -1;
+
+    if(index < 0)
+    {
+        qWarning() << "Could not connect slot:" << slot;
+    }
+
+    return metaObject->method(index);
+}
+
+} // anonymous
 
 namespace qpdfview
 {
@@ -40,15 +65,14 @@ void GraphicsCompositionModeEffect::draw(QPainter* painter)
     painter->save();
 
     painter->setCompositionMode(m_compositionMode);
+
     drawSource(painter);
 
     painter->restore();
-
 }
 
 TabBar::TabBar(QWidget* parent) : QTabBar(parent)
 {
-    setContextMenuPolicy(Qt::CustomContextMenu);
 }
 
 QSize TabBar::tabSizeHint(int index) const
@@ -57,7 +81,7 @@ QSize TabBar::tabSizeHint(int index) const
 
     const TabWidget* tabWidget = qobject_cast< TabWidget* >(parentWidget());
 
-    if(tabWidget->spreadTabs())
+    if(tabWidget != 0 && tabWidget->spreadTabs())
     {
         switch(tabWidget->tabPosition())
         {
@@ -92,6 +116,8 @@ TabWidget::TabWidget(QWidget* parent) : QTabWidget(parent),
 {
     setTabBar(new TabBar(this));
 
+    tabBar()->setContextMenuPolicy(Qt::CustomContextMenu);
+
     connect(tabBar(), SIGNAL(customContextMenuRequested(QPoint)), SLOT(on_tabBar_customContextMenuRequested(QPoint)));
 }
 
@@ -125,13 +151,16 @@ bool TabWidget::spreadTabs() const
 
 void TabWidget::setSpreadTabs(bool spreadTabs)
 {
-    m_spreadTabs = spreadTabs;
+    if(m_spreadTabs != spreadTabs)
+    {
+        m_spreadTabs = spreadTabs;
 
-    QResizeEvent resizeEvent(tabBar()->size(), tabBar()->size());
-    QApplication::sendEvent(tabBar(), &resizeEvent);
+        QResizeEvent resizeEvent(tabBar()->size(), tabBar()->size());
+        QApplication::sendEvent(tabBar(), &resizeEvent);
+    }
 }
 
-void TabWidget::on_tabBar_customContextMenuRequested(const QPoint &pos)
+void TabWidget::on_tabBar_customContextMenuRequested(const QPoint& pos)
 {
     emit tabContextMenuRequested(tabBar()->mapToGlobal(pos), tabBar()->tabAt(pos));
 }
@@ -294,6 +323,93 @@ void SpinBox::keyPressEvent(QKeyEvent* event)
     }
 }
 
+MappingSpinBox::MappingSpinBox(QObject* mapper, const char* textFromValue, const char* valueFromText, QWidget* parent) : SpinBox(parent),
+    m_mapper(mapper)
+{
+    const QMetaObject* metaObject = mapper->metaObject();
+
+    m_textFromValue = slotToMethod(metaObject, textFromValue);
+    m_valueFromText = slotToMethod(metaObject, valueFromText);
+}
+
+QString MappingSpinBox::textFromValue(int val) const
+{
+    QString text;
+    bool ok = false;
+
+    const bool mapped = m_textFromValue.invoke(m_mapper, Qt::DirectConnection,
+                                               Q_RETURN_ARG(QString, text), Q_ARG(int, val), Q_ARG(bool*, &ok));
+
+    if(!mapped || !ok)
+    {
+        text = SpinBox::textFromValue(val);
+    }
+
+    return text;
+}
+
+int MappingSpinBox::valueFromText(const QString& text) const
+{
+    int val;
+    bool ok = false;
+
+    const bool mapped = m_valueFromText.invoke(m_mapper, Qt::DirectConnection,
+                                               Q_RETURN_ARG(int, val), Q_ARG(QString, text), Q_ARG(bool*, &ok));
+
+    if(!mapped || !ok)
+    {
+        val = SpinBox::valueFromText(text);
+    }
+
+    return val;
+}
+
+QValidator::State MappingSpinBox::validate(QString& input, int& pos) const
+{
+    Q_UNUSED(input);
+    Q_UNUSED(pos);
+
+    return QValidator::Acceptable;
+}
+
+int getMappedNumber(QObject* mapper, const char* textFromValue, const char* valueFromText,
+                    QWidget* parent, const QString& title, const QString& caption,
+                    int value, int min, int max, bool* ok, Qt::WindowFlags flags)
+{
+    QDialog* dialog = new QDialog(parent, flags | Qt::MSWindowsFixedSizeDialogHint);
+    dialog->setWindowTitle(title);
+
+    QLabel* label = new QLabel(dialog);
+    label->setText(caption);
+
+    MappingSpinBox* mappingSpinBox = new MappingSpinBox(mapper, textFromValue, valueFromText, dialog);
+    mappingSpinBox->setRange(min, max);
+    mappingSpinBox->setValue(value);
+
+    QDialogButtonBox* dialogButtonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel, Qt::Horizontal, dialog);
+    QObject::connect(dialogButtonBox, SIGNAL(accepted()), dialog, SLOT(accept()));
+    QObject::connect(dialogButtonBox, SIGNAL(rejected()), dialog, SLOT(reject()));
+
+    dialog->setLayout(new QVBoxLayout(dialog));
+    dialog->layout()->addWidget(label);
+    dialog->layout()->addWidget(mappingSpinBox);
+    dialog->layout()->addWidget(dialogButtonBox);
+
+    dialog->setFocusProxy(mappingSpinBox);
+
+    const int dialogResult = dialog->exec();
+    const int number = mappingSpinBox->value();
+
+    delete dialog;
+
+    if(ok)
+    {
+        *ok = dialogResult == QDialog::Accepted;
+    }
+
+    return number;
+}
+
 ProgressLineEdit::ProgressLineEdit(QWidget* parent) : QLineEdit(parent),
     m_progress(0)
 {    
@@ -334,8 +450,6 @@ void ProgressLineEdit::keyPressEvent(QKeyEvent* event)
     }
 }
 
-// search line edit
-
 SearchLineEdit::SearchLineEdit(QWidget* parent) : ProgressLineEdit(parent)
 {
     m_timer = new QTimer(this);
