@@ -1,5 +1,6 @@
 /*
 
+Copyright 2014 S. Razi Alavizadeh
 Copyright 2012-2014 Adam Reichold
 Copyright 2014 Dorian Scholz
 
@@ -26,6 +27,7 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <QComboBox>
 #include <QGraphicsEffect>
 #include <QLineEdit>
+#include <QMetaMethod>
 #include <QPainter>
 #include <QSpinBox>
 #include <QTreeView>
@@ -191,6 +193,35 @@ private:
 
 };
 
+// mapping spin box
+
+class MappingSpinBox : public SpinBox
+{
+    Q_OBJECT
+
+public:
+    MappingSpinBox(QObject* mapper, const char* textFromValue, const char* valueFromText, QWidget* parent = 0);
+
+protected:
+    QString textFromValue(int val) const;
+    int valueFromText(const QString& text) const;
+
+    QValidator::State validate(QString& input, int& pos) const;
+
+private:
+    Q_DISABLE_COPY(MappingSpinBox)
+
+    QObject* m_mapper;
+    QMetaMethod m_textFromValue;
+    QMetaMethod m_valueFromText;
+
+};
+
+int getMappedNumber(QObject* mapper, const char* textFromValue, const char* valueFromText,
+                    QWidget* parent, const QString& title, const QString& caption,
+                    int value = 0, int min = -2147483647, int max = 2147483647,
+                    bool* ok = 0, Qt::WindowFlags flags = 0);
+
 // progress line edit
 
 class ProgressLineEdit : public QLineEdit
