@@ -31,6 +31,7 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <QMenu>
 #include <QMessageBox>
 #include <QPainter>
+#include <QStyleOptionGraphicsItem>
 #include <QTimer>
 #include <QToolTip>
 #include <QUrl>
@@ -38,59 +39,66 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include "settings.h"
 #include "model.h"
 #include "rendertask.h"
+#include "tileitem.h"
+
+namespace
+{
+
+using namespace qpdfview;
+
+const qreal proxyPadding = 2.0;
+
+bool modifiersUseMouseButton(Settings* settings, Qt::MouseButton mouseButton)
+{
+    return ((settings->pageItem().copyToClipboardModifiers() | settings->pageItem().addAnnotationModifiers()) & mouseButton) != 0;
+}
+
+} // anonymous
 
 namespace qpdfview
 {
 
 Settings* PageItem::s_settings = 0;
 
-QCache< PageItem*, QPixmap > PageItem::s_cache;
-
 PageItem::PageItem(Model::Page* page, int index, bool presentationMode, QGraphicsItem* parent) : QGraphicsObject(parent),
-    m_page(0),
-    m_index(-1),
-    m_size(),
+    m_page(page),
+    m_size(page->size()),
+    m_cropRect(),
+    m_index(index),
+    m_presentationMode(presentationMode),
+    m_highlights(),
     m_links(),
     m_annotations(),
     m_formFields(),
-    m_annotationOverlay(),
-    m_formFieldOverlay(),
-    m_presentationMode(presentationMode),
-    m_invertColors(false),
-    m_highlights(),
     m_rubberBandMode(ModifiersMode),
     m_rubberBand(),
-    m_resolutionX(72),
-    m_resolutionY(72),
-    m_devicePixelRatio(1.0),
-    m_scaleFactor(1.0),
-    m_rotation(RotateBy0),
+    m_annotationOverlay(),
+    m_formFieldOverlay(),
+    m_renderParam(),
     m_transform(),
     m_normalizedTransform(),
     m_boundingRect(),
-    m_pixmap(),
-    m_renderTask(0),
-    m_obsoletePixmap(),
-    m_obsoleteTransform()
+    m_tileItems()
 {
     if(s_settings == 0)
     {
         s_settings = Settings::instance();
     }
 
-    s_cache.setMaxCost(s_settings->pageItem().cacheSize());
-
     setAcceptHoverEvents(true);
 
-    m_renderTask = new RenderTask(this);
+    setFlag(QGraphicsItem::ItemUsesExtendedStyleOption, s_settings->pageItem().useTiling());
+    setFlag(QGraphicsItem::ItemClipsToShape, s_settings->pageItem().trimMargins());
 
-    connect(m_renderTask, SIGNAL(finished()), SLOT(on_renderTask_finished()));
-    connect(m_renderTask, SIGNAL(imageReady(int,int,qreal,qreal,Rotation,bool,bool,QImage)), SLOT(on_renderTask_imageReady(int,int,qreal,qreal,Rotation,bool,bool,QImage)));
+    if(!s_settings->pageItem().useTiling())
+    {
+        TileItem* tile = new TileItem(this);
 
-    m_page = page;
+        m_tileItems.resize(1);
+        m_tileItems.squeeze();
 
-    m_index = index;
-    m_size = m_page->size();
+        m_tileItems.replace(0, tile);
+    }
 
     QTimer::singleShot(0, this, SLOT(loadInteractiveElements()));
 
@@ -102,10 +110,7 @@ PageItem::~PageItem()
     hideAnnotationOverlay(false);
     hideFormFieldOverlay(false);
 
-    m_renderTask->cancel();
-    m_renderTask->wait();
-
-    s_cache.remove(this);
+    TileItem::dropCachedPixmaps(this);
 
     qDeleteAll(m_links);
     qDeleteAll(m_annotations);
@@ -114,12 +119,24 @@ PageItem::~PageItem()
 
 QRectF PageItem::boundingRect() const
 {
-    return m_boundingRect;
+    if(m_cropRect.isNull())
+    {
+        return m_boundingRect;
+    }
+
+    QRectF boundingRect;
+
+    boundingRect.setLeft(m_boundingRect.left() + m_cropRect.left() * m_boundingRect.width());
+    boundingRect.setTop(m_boundingRect.top() + m_cropRect.top() * m_boundingRect.height());
+    boundingRect.setWidth(m_cropRect.width() * m_boundingRect.width());
+    boundingRect.setHeight(m_cropRect.height() * m_boundingRect.height());
+
+    return boundingRect;
 }
 
-void PageItem::paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*)
+void PageItem::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget*)
 {
-    paintPage(painter, cachedPixmap());
+    paintPage(painter, option->exposedRect);
 
     paintLinks(painter);
     paintFormFields(painter);
@@ -128,11 +145,38 @@ void PageItem::paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget
     paintRubberBand(painter);
 }
 
-void PageItem::setInvertColors(bool invertColors)
+qreal PageItem::displayedWidth() const
 {
-    m_invertColors = invertColors;
+    const qreal cropWidth = m_cropRect.isNull() ? 1.0 : m_cropRect.width();
+    const qreal cropHeight = m_cropRect.isNull() ? 1.0 : m_cropRect.height();
 
-    refresh();
+    switch(m_renderParam.rotation)
+    {
+    default:
+    case RotateBy0:
+    case RotateBy180:
+        return m_renderParam.resolution.resolutionX / 72.0 * cropWidth * m_size.width();
+    case RotateBy90:
+    case RotateBy270:
+        return m_renderParam.resolution.resolutionX / 72.0 * cropHeight * m_size.height();
+    }
+}
+
+qreal PageItem::displayedHeight() const
+{
+    const qreal cropHeight = m_cropRect.isNull() ? 1.0 : m_cropRect.height();
+    const qreal cropWidth = m_cropRect.isNull() ? 1.0 : m_cropRect.width();
+
+    switch(m_renderParam.rotation)
+    {
+    default:
+    case RotateBy0:
+    case RotateBy180:
+        return m_renderParam.resolution.resolutionY / 72.0 * cropHeight * m_size.height();
+    case RotateBy90:
+    case RotateBy270:
+        return m_renderParam.resolution.resolutionY / 72.0 * cropWidth * m_size.width();
+    }
 }
 
 void PageItem::setHighlights(const QList< QRectF >& highlights)
@@ -161,12 +205,12 @@ void PageItem::setRubberBandMode(RubberBandMode rubberBandMode)
 
 void PageItem::setResolution(int resolutionX, int resolutionY)
 {
-    if((m_resolutionX != resolutionX || m_resolutionY != resolutionY) && resolutionX > 0 && resolutionY > 0)
+    if((m_renderParam.resolution.resolutionX != resolutionX || m_renderParam.resolution.resolutionY != resolutionY) && resolutionX > 0 && resolutionY > 0)
     {
-        refresh();
+        refresh(true);
 
-        m_resolutionX = resolutionX;
-        m_resolutionY = resolutionY;
+        m_renderParam.resolution.resolutionX = resolutionX;
+        m_renderParam.resolution.resolutionY = resolutionY;
 
         prepareGeometryChange();
         prepareGeometry();
@@ -175,11 +219,16 @@ void PageItem::setResolution(int resolutionX, int resolutionY)
 
 void PageItem::setDevicePixelRatio(qreal devicePixelRatio)
 {
-    if(!qFuzzyCompare(m_devicePixelRatio, devicePixelRatio) && devicePixelRatio > 0.0)
+    if(!s_settings->pageItem().useDevicePixelRatio())
     {
-        refresh();
+        return;
+    }
 
-        m_devicePixelRatio = devicePixelRatio;
+    if(!qFuzzyCompare(m_renderParam.resolution.devicePixelRatio, devicePixelRatio) && devicePixelRatio > 0.0)
+    {
+        refresh(true);
+
+        m_renderParam.resolution.devicePixelRatio = devicePixelRatio;
 
         prepareGeometryChange();
         prepareGeometry();
@@ -188,11 +237,11 @@ void PageItem::setDevicePixelRatio(qreal devicePixelRatio)
 
 void PageItem::setScaleFactor(qreal scaleFactor)
 {
-    if(!qFuzzyCompare(m_scaleFactor, scaleFactor) && scaleFactor > 0.0)
+    if(!qFuzzyCompare(m_renderParam.scaleFactor, scaleFactor) && scaleFactor > 0.0)
     {
-        refresh();
+        refresh(true);
 
-        m_scaleFactor = scaleFactor;
+        m_renderParam.scaleFactor = scaleFactor;
 
         prepareGeometryChange();
         prepareGeometry();
@@ -201,100 +250,87 @@ void PageItem::setScaleFactor(qreal scaleFactor)
 
 void PageItem::setRotation(Rotation rotation)
 {
-    if(m_rotation != rotation && rotation >= 0 && rotation < NumberOfRotations)
+    if(m_renderParam.rotation != rotation && rotation >= 0 && rotation < NumberOfRotations)
     {
-        refresh();
+        refresh(false);
 
-        m_rotation = rotation;
+        m_renderParam.rotation = rotation;
 
         prepareGeometryChange();
         prepareGeometry();
     }
 }
 
-void PageItem::refresh()
+void PageItem::setInvertColors(bool invertColors)
 {
-    m_renderTask->cancel();
-
-    if(s_settings->pageItem().keepObsoletePixmaps() && s_cache.contains(this))
+    if(m_renderParam.invertColors != invertColors)
     {
-        m_obsoletePixmap = *s_cache.object(this);
-        m_obsoleteTopLeft = m_boundingRect.topLeft();
-        m_obsoleteTransform = m_transform.inverted();
+        refresh(false);
+
+        m_renderParam.invertColors = invertColors;
+    }
+}
+
+
+void PageItem::refresh(bool keepObsoletePixmaps, bool dropCachedPixmaps)
+{
+    if(!s_settings->pageItem().useTiling())
+    {
+        m_tileItems.first()->refresh(keepObsoletePixmaps);
+    }
+    else
+    {
+        foreach(TileItem* tile, m_tileItems)
+        {
+            tile->refresh(keepObsoletePixmaps);
+        }
     }
 
-    m_pixmap = QPixmap();
-    s_cache.remove(this);
+    if(!keepObsoletePixmaps)
+    {
+        m_cropRect = QRectF();
+    }
+
+    if(dropCachedPixmaps)
+    {
+        TileItem::dropCachedPixmaps(this);
+    }
 
     update();
 }
 
-void PageItem::startRender(bool prefetch)
+int PageItem::startRender(bool prefetch)
 {
-    if(prefetch && s_cache.contains(this))
+    int cost = 0;
+
+    if(!s_settings->pageItem().useTiling())
     {
-        return;
+        cost += m_tileItems.first()->startRender(prefetch);
+    }
+    else
+    {
+        foreach(TileItem* tile, m_tileItems)
+        {
+            cost += tile->startRender(prefetch);
+        }
     }
 
-    if(!m_renderTask->isRunning())
-    {
-        m_renderTask->start(m_page, m_resolutionX, m_resolutionY, effectiveDevicePixelRatio(), m_scaleFactor, m_rotation, m_invertColors, prefetch);
-    }
+    return cost;
 }
 
 void PageItem::cancelRender()
 {
-    m_renderTask->cancel();
-
-    m_pixmap = QPixmap();
-    m_obsoletePixmap = QPixmap();
-}
-
-void PageItem::on_renderTask_finished()
-{
-    update();
-}
-
-void PageItem::on_renderTask_imageReady(int resolutionX, int resolutionY, qreal devicePixelRatio, qreal scaleFactor, Rotation rotation, bool invertColors, bool prefetch, QImage image)
-{
-    if(m_resolutionX != resolutionX || m_resolutionY != resolutionY
-            || !qFuzzyCompare(effectiveDevicePixelRatio(), devicePixelRatio)
-            || !qFuzzyCompare(m_scaleFactor, scaleFactor) || m_rotation != rotation
-            || m_invertColors != invertColors)
+    if(!s_settings->pageItem().useTiling())
     {
-        return;
-    }
-
-    if(image.isNull())
-    {
-        // error icon
-
-        const qreal extent = qMin(0.1 * m_boundingRect.width(), 0.1 * m_boundingRect.height());
-        const QRectF rect(0.01 * m_boundingRect.width(), 0.01 * m_boundingRect.height(), extent, extent);
-
-        image = QImage(qFloor(0.01 * m_boundingRect.width() + extent), qFloor(0.01 * m_boundingRect.height() + extent), QImage::Format_ARGB32);
-        image.fill(Qt::transparent);
-
-        QPainter painter(&image);
-        s_settings->pageItem().errorIcon().paint(&painter, rect.toRect());
-    }
-
-    if(prefetch)
-    {
-        QPixmap* pixmap = new QPixmap(QPixmap::fromImage(image));
-
-        int cost = pixmap->width() * pixmap->height() * pixmap->depth() / 8;
-        s_cache.insert(this, pixmap, cost);
+        m_tileItems.first()->cancelRender();
     }
     else
     {
-        if(!m_renderTask->wasCanceled())
+        foreach(TileItem* tile, m_tileItems)
         {
-            m_pixmap = QPixmap::fromImage(image);
+            tile->cancelRender();
         }
     }
-
-    m_obsoletePixmap = QPixmap();
 }
 
 void PageItem::showAnnotationOverlay(Model::Annotation* selectedAnnotation)
@@ -431,21 +467,25 @@ void PageItem::hoverLeaveEvent(QGraphicsSceneHoverEvent*)
 
 void PageItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
 {
+    const Qt::KeyboardModifiers copyToClipboardModifiers = s_settings->pageItem().copyToClipboardModifiers();
+    const Qt::KeyboardModifiers addAnnotationModifiers = s_settings->pageItem().addAnnotationModifiers();
+
+    const bool copyToClipboardModifiersActive = event->modifiers() == copyToClipboardModifiers || ((event->buttons() & copyToClipboardModifiers) != 0);
+    const bool addAnnotationModifiersActive = event->modifiers() == addAnnotationModifiers || ((event->buttons() & addAnnotationModifiers) != 0);
+
     // rubber band
 
     if(m_rubberBandMode == ModifiersMode && !m_presentationMode
-            && (event->modifiers() == Qt::NoModifier
-                || event->modifiers() == s_settings->pageItem().copyToClipboardModifiers()
-                || event->modifiers() == s_settings->pageItem().addAnnotationModifiers())
+            && (event->modifiers() == Qt::NoModifier || copyToClipboardModifiersActive || addAnnotationModifiersActive)
             && (event->button() == Qt::LeftButton || event->button() == Qt::MidButton))
     {
         setCursor(Qt::CrossCursor);
 
-        if(event->modifiers() == s_settings->pageItem().copyToClipboardModifiers() && event->button() == Qt::LeftButton)
+        if(copyToClipboardModifiersActive && event->button() == Qt::LeftButton)
         {
             m_rubberBandMode = CopyToClipboardMode;
         }
-        else if(event->modifiers() == s_settings->pageItem().addAnnotationModifiers() && event->button() == Qt::LeftButton)
+        else if(addAnnotationModifiersActive && event->button() == Qt::LeftButton)
         {
             m_rubberBandMode = AddAnnotationMode;
         }
@@ -608,19 +648,38 @@ void PageItem::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
 
 void PageItem::contextMenuEvent(QGraphicsSceneContextMenuEvent* event)
 {
+    if(event->reason() == QGraphicsSceneContextMenuEvent::Mouse && modifiersUseMouseButton(s_settings, Qt::RightButton))
+    {
+        event->accept();
+        return;
+    }
+
     if(m_presentationMode)
     {
         event->ignore();
         return;
     }
 
+    foreach(Model::Link* link, m_links)
+    {
+        if(m_normalizedTransform.map(link->boundary).contains(event->pos()))
+        {
+            unsetCursor();
+
+            showLinkContextMenu(link, event->screenPos());
+
+            event->accept();
+            return;
+        }
+    }
+
     foreach(Model::Annotation* annotation, m_annotations)
     {
         if(m_normalizedTransform.mapRect(annotation->boundary()).contains(event->pos()))
         {
             unsetCursor();
 
-            removeAnnotation(annotation, event->screenPos());
+            showAnnotationContextMenu(annotation, event->screenPos());
 
             event->accept();
             return;
@@ -654,39 +713,78 @@ void PageItem::loadInteractiveElements()
     update();
 }
 
+void PageItem::updateCropRect()
+{
+    QRectF cropRect;
+
+    if(!s_settings->pageItem().useTiling())
+    {
+        cropRect = m_tileItems.first()->cropRect();
+    }
+    else
+    {
+        foreach(TileItem* tile, m_tileItems)
+        {
+            const QRect& tileRect = tile->rect();
+            const QRectF& tileCropRect = tile->cropRect();
+
+            if(tileCropRect.isNull())
+            {
+                cropRect = QRectF();
+                break;
+            }
+
+            const qreal left = (tileRect.left() + tileCropRect.left() * tileRect.width()) / m_boundingRect.width();
+            const qreal top = (tileRect.top() + tileCropRect.top() * tileRect.height()) / m_boundingRect.height();
+            const qreal width = tileCropRect.width() * tileRect.width() / m_boundingRect.width();
+            const qreal height = tileCropRect.height() * tileRect.height() / m_boundingRect.height();
+
+            cropRect = cropRect.united(QRectF(left, top, width, height));
+        }
+    }
+
+    if(m_cropRect.isNull() && !cropRect.isNull())
+    {
+        m_cropRect = cropRect;
+
+        prepareGeometryChange();
+        emit cropRectChanged();
+    }
+}
+
 void PageItem::copyToClipboard(const QPoint& screenPos)
 {
     QMenu menu;
 
-    const QAction* copyTextAction = menu.addAction(tr("Copy &text"));
-    QAction* useTextAsSelectionAction = menu.addAction(tr("Use text as &selection"));
+    QAction* copyTextAction = menu.addAction(tr("Copy &text"));
+    QAction* selectTextAction = menu.addAction(tr("&Select text"));
     const QAction* copyImageAction = menu.addAction(tr("Copy &image"));
     const QAction* saveImageToFileAction = menu.addAction(tr("Save image to &file..."));
 
-    useTextAsSelectionAction->setVisible(QApplication::clipboard()->supportsSelection());
+    const QString text = m_page->text(m_transform.inverted().mapRect(m_rubberBand));
+
+    copyTextAction->setVisible(!text.isEmpty());
+    selectTextAction->setVisible(!text.isEmpty() && QApplication::clipboard()->supportsSelection());
 
     const QAction* action = menu.exec(screenPos);
 
-    if(action == copyTextAction || action == useTextAsSelectionAction)
+    if(action == copyTextAction || action == selectTextAction)
     {
-        const QString text = m_page->text(m_transform.inverted().mapRect(m_rubberBand));
-
-        if(!text.isEmpty())
+        if(action == copyTextAction)
         {
-            if(action == copyTextAction)
-            {
-                QApplication::clipboard()->setText(text);
-            }
-            else
-            {
-                QApplication::clipboard()->setText(text, QClipboard::Selection);
-            }
+            QApplication::clipboard()->setText(text);
+        }
+        else
+        {
+            QApplication::clipboard()->setText(text, QClipboard::Selection);
         }
     }
     else if(action == copyImageAction || action == saveImageToFileAction)
     {
         const QRect rect = m_rubberBand.translated(-m_boundingRect.topLeft()).toRect();
-        const QImage image = s_cache.contains(this) ? s_cache.object(this)->copy(rect).toImage() : m_page->render(m_resolutionX * m_scaleFactor, m_scaleFactor * m_resolutionY, m_rotation, rect);
+        const QImage image = m_page->render(m_renderParam.resolution.resolutionX * m_renderParam.scaleFactor,
+                                            m_renderParam.resolution.resolutionY * m_renderParam.scaleFactor,
+                                            m_renderParam.rotation, rect);
 
         if(!image.isNull())
         {
@@ -739,7 +837,7 @@ void PageItem::addAnnotation(const QPoint& screenPos)
             m_annotations.append(annotation);
             connect(annotation, SIGNAL(wasModified()), SIGNAL(wasModified()));
 
-            refresh();
+            refresh(false, true);
             emit wasModified();
 
             showAnnotationOverlay(annotation);
@@ -747,7 +845,31 @@ void PageItem::addAnnotation(const QPoint& screenPos)
     }
 }
 
-void PageItem::removeAnnotation(Model::Annotation* annotation, const QPoint& screenPos)
+void PageItem::showLinkContextMenu(Model::Link* link, const QPoint& screenPos)
+{
+    if(link->page == -1)
+    {
+        QMenu menu;
+
+        const QAction* copyLinkAddressAction = menu.addAction(tr("&Copy link address"));
+        QAction* selectLinkAddressAction = menu.addAction(tr("&Select link address"));
+
+        selectLinkAddressAction->setVisible(QApplication::clipboard()->supportsSelection());
+
+        const QAction* action = menu.exec(screenPos);
+
+        if(action == copyLinkAddressAction)
+        {
+            QApplication::clipboard()->setText(link->urlOrFileName);
+        }
+        else if(action == selectLinkAddressAction)
+        {
+            QApplication::clipboard()->setText(link->urlOrFileName, QClipboard::Selection);
+        }
+    }
+}
+
+void PageItem::showAnnotationContextMenu(Model::Annotation* annotation, const QPoint& screenPos)
 {
     if(m_page->canAddAndRemoveAnnotations())
     {
@@ -764,7 +886,7 @@ void PageItem::removeAnnotation(Model::Annotation* annotation, const QPoint& scr
 
             annotation->deleteLater();
 
-            refresh();
+            refresh(false, true);
             emit wasModified();
         }
     }
@@ -808,15 +930,14 @@ void PageItem::addProxy(Overlay& overlay, const char* hideOverlay, Element* elem
 template< typename Overlay >
 void PageItem::hideOverlay(Overlay& overlay, bool deleteLater)
 {
-    Overlay discardedOverlay;
-
 #if QT_VERSION >= QT_VERSION_CHECK(4,8,0)
 
+    Overlay discardedOverlay;
     discardedOverlay.swap(overlay);
 
 #else
 
-    discardedOverlay = overlay;
+    Overlay discardedOverlay(overlay);
     overlay = Overlay();
 
 #endif // QT_VERSION
@@ -835,7 +956,7 @@ void PageItem::hideOverlay(Overlay& overlay, bool deleteLater)
             }
         }
 
-        refresh();
+        refresh(false, true);
     }
 }
 
@@ -875,7 +996,7 @@ void PageItem::setProxyGeometry(Model::FormField* formField, QGraphicsProxyWidge
     qreal width = rect.width();
     qreal height = rect.height();
 
-    switch(m_rotation)
+    switch(m_renderParam.rotation)
     {
     default:
     case RotateBy0:
@@ -901,109 +1022,126 @@ void PageItem::setProxyGeometry(Model::FormField* formField, QGraphicsProxyWidge
         break;
     }
 
-    width /= m_scaleFactor;
-    height /= m_scaleFactor;
+    width /= m_renderParam.scaleFactor;
+    height /= m_renderParam.scaleFactor;
 
-    proxy->setScale(m_scaleFactor);
+    proxy->setScale(m_renderParam.scaleFactor);
 
     proxy->setGeometry(QRectF(x - proxyPadding, y - proxyPadding, width + proxyPadding, height + proxyPadding));
 }
 
-qreal PageItem::effectiveDevicePixelRatio()
-{
-#if QT_VERSION >= QT_VERSION_CHECK(5,1,0)
-
-    return s_settings->pageItem().useDevicePixelRatio() ? m_devicePixelRatio : 1.0;
-
-#else
-
-    return 1.0;
-
-#endif // QT_VERSION
-}
-
 void PageItem::prepareGeometry()
 {
     m_transform.reset();
-    m_normalizedTransform.reset();
 
-    switch(m_rotation)
+    m_transform.scale(m_renderParam.resolution.resolutionX * m_renderParam.scaleFactor / 72.0,
+                      m_renderParam.resolution.resolutionY * m_renderParam.scaleFactor / 72.0);
+
+    switch(m_renderParam.rotation)
     {
     default:
     case RotateBy0:
         break;
     case RotateBy90:
         m_transform.rotate(90.0);
-        m_normalizedTransform.rotate(90.0);
         break;
     case RotateBy180:
         m_transform.rotate(180.0);
-        m_normalizedTransform.rotate(180.0);
         break;
     case RotateBy270:
         m_transform.rotate(270.0);
-        m_normalizedTransform.rotate(270.0);
         break;
     }
 
-    switch(m_rotation)
-    {
-    default:
-    case RotateBy0:
-    case RotateBy180:
-        m_transform.scale(m_scaleFactor * m_resolutionX / 72.0, m_scaleFactor * m_resolutionY / 72.0);
-        m_normalizedTransform.scale(m_scaleFactor * m_resolutionX / 72.0 * m_size.width(), m_scaleFactor * m_resolutionY / 72.0 * m_size.height());
-        break;
-    case RotateBy90:
-    case RotateBy270:
-        m_transform.scale(m_scaleFactor * m_resolutionY / 72.0, m_scaleFactor * m_resolutionX / 72.0);
-        m_normalizedTransform.scale(m_scaleFactor * m_resolutionY / 72.0 * m_size.width(), m_scaleFactor * m_resolutionX / 72.0 * m_size.height());
-        break;
-    }
+    m_normalizedTransform = m_transform;
+    m_normalizedTransform.scale(m_size.width(), m_size.height());
+
 
     m_boundingRect = m_transform.mapRect(QRectF(QPointF(), m_size));
 
     m_boundingRect.setWidth(qRound(m_boundingRect.width()));
     m_boundingRect.setHeight(qRound(m_boundingRect.height()));
 
+
+    prepareTiling();
+
     updateAnnotationOverlay();
     updateFormFieldOverlay();
 }
 
-QPixmap PageItem::cachedPixmap()
+void PageItem::prepareTiling()
 {
-    QPixmap pixmap;
-
-    if(s_cache.contains(this))
+    if(!s_settings->pageItem().useTiling())
     {
-        pixmap = *s_cache.object(this);
+        m_tileItems.first()->setRect(QRect(0, 0, m_boundingRect.width(), m_boundingRect.height()));
+
+        return;
     }
-    else
-    {
-        if(!m_pixmap.isNull())
-        {
-            pixmap = m_pixmap;
-            m_pixmap = QPixmap();
 
-            int cost = pixmap.width() * pixmap.height() * pixmap.depth() / 8;
-            s_cache.insert(this, new QPixmap(pixmap), cost);
-        }
-        else
+
+    const qreal pageWidth = m_boundingRect.width();
+    const qreal pageHeight = m_boundingRect.height();
+
+    const int tileSize = s_settings->pageItem().tileSize();
+
+    int tileWidth = pageWidth < pageHeight ? tileSize * pageWidth / pageHeight : tileSize;
+    int tileHeight = pageHeight < pageWidth ? tileSize * pageHeight / pageWidth : tileSize;
+
+    const int columnCount = qCeil(pageWidth / tileWidth);
+    const int rowCount = qCeil(pageHeight / tileHeight);
+
+    tileWidth = qCeil(pageWidth / columnCount);
+    tileHeight = qCeil(pageHeight / rowCount);
+
+
+    const int newCount = columnCount * rowCount;
+    const int oldCount = m_tileItems.count();
+
+    for(int index = newCount; index < oldCount; ++index)
+    {
+        m_tileItems.at(index)->deleteAfterRender();
+    }
+
+    m_tileItems.resize(newCount);
+
+    for(int index = oldCount; index < newCount; ++index)
+    {
+        m_tileItems.replace(index, new TileItem(this));
+    }
+
+    if(oldCount != newCount)
+    {
+        foreach(TileItem* tile, m_tileItems)
         {
-            startRender();
+            tile->dropObsoletePixmap();
         }
     }
 
-    return pixmap;
+
+    for(int column = 0; column < columnCount; ++column)
+    {
+        for(int row = 0; row < rowCount; ++row)
+        {
+            const int left = column > 0 ? column * tileWidth : 0.0;
+            const int top = row > 0 ? row * tileHeight : 0.0;
+
+            const int width = column < (columnCount - 1) ? tileWidth : pageWidth - left;
+            const int height = row < (rowCount - 1) ? tileHeight : pageHeight - top;
+
+            m_tileItems.at(column * rowCount + row)->setRect(QRect(left, top, width, height));
+        }
+    }
 }
 
-void PageItem::paintPage(QPainter* painter, const QPixmap& pixmap) const
+void PageItem::paintPage(QPainter* painter, const QRectF& exposedRect) const
 {
     if(s_settings->pageItem().decoratePages() && !m_presentationMode)
     {
+        // background
+
         QColor paperColor = s_settings->pageItem().paperColor();
 
-        if(m_invertColors)
+        if(m_renderParam.invertColors)
         {
             paperColor.setRgb(~paperColor.rgb());
         }
@@ -1011,34 +1149,47 @@ void PageItem::paintPage(QPainter* painter, const QPixmap& pixmap) const
         painter->fillRect(m_boundingRect, QBrush(paperColor));
     }
 
-    if(!pixmap.isNull())
+    // tiles
+
+    if(!s_settings->pageItem().useTiling())
     {
-        painter->drawPixmap(m_boundingRect.topLeft(), pixmap);
-    }
-    else if(!m_obsoletePixmap.isNull())
-    {
-        painter->save();
-
-        painter->setTransform(m_obsoleteTransform, true);
-        painter->setTransform(m_transform, true);
-
-        painter->drawPixmap(m_obsoleteTopLeft, m_obsoletePixmap);
-
-        painter->restore();
+        m_tileItems.first()->paint(painter, m_boundingRect.topLeft());
     }
     else
     {
-        // progess icon
+        const QRectF& translatedExposedRect = exposedRect.translated(-m_boundingRect.topLeft());
 
-        const qreal extent = qMin(0.1 * m_boundingRect.width(), 0.1 * m_boundingRect.height());
-        const QRectF rect(m_boundingRect.left() + 0.01 * m_boundingRect.width(), m_boundingRect.top() + 0.01 * m_boundingRect.height(), extent, extent);
-
-        s_settings->pageItem().progressIcon().paint(painter, rect.toRect());
+        foreach(TileItem* tile, m_tileItems)
+        {
+            if(translatedExposedRect.intersects(tile->rect()))
+            {
+                tile->paint(painter, m_boundingRect.topLeft());
+            }
+            else
+            {
+                tile->cancelRender();
+            }
+        }
     }
 
     if(s_settings->pageItem().decoratePages() && !m_presentationMode)
     {
-        painter->drawRect(m_boundingRect);
+        // border
+
+        if(!s_settings->pageItem().trimMargins())
+        {
+            painter->drawRect(m_boundingRect);
+        }
+        else
+        {
+            painter->save();
+
+            painter->setClipping(false);
+
+            painter->drawRect(boundingRect());
+
+            painter->restore();
+        }
     }
 }
 
@@ -1113,152 +1264,4 @@ void PageItem::paintRubberBand(QPainter* painter) const
     }
 }
 
-ThumbnailItem::ThumbnailItem(Model::Page* page, int index, QGraphicsItem* parent) : PageItem(page, index, false, parent),
-#if QT_VERSION >= QT_VERSION_CHECK(4,7,0)
-    m_text(QString::number(index + 1)),
-#endif // QT_VERSION
-    m_current(false)
-{
-    setAcceptHoverEvents(false);
-}
-
-QRectF ThumbnailItem::boundingRect() const
-{
-#if QT_VERSION >= QT_VERSION_CHECK(4,7,0)
-
-    return PageItem::boundingRect().adjusted(0.0, 0.0, 0.0, 2.0 * m_text.size().height());
-
-#else
-
-    return PageItem::boundingRect().adjusted(0.0, 0.0, 0.0, 2.0 * QFontMetrics(QFont()).height());
-
-#endif // QT_VERSION
-}
-
-void ThumbnailItem::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
-{
-    PageItem::paint(painter, option, widget);
-
-    const QRectF boundingRect = PageItem::boundingRect();
-
-#if QT_VERSION >= QT_VERSION_CHECK(4,7,0)
-
-    const QSizeF textSize = m_text.size();
-
-    QPointF pos = boundingRect.bottomLeft();
-    pos.rx() += 0.5 * (boundingRect.width() - textSize.width());
-    pos.ry() += 0.5 * textSize.height();
-
-    painter->drawStaticText(pos, m_text);
-
-#else
-
-    const QString text = QString::number(index() + 1);
-    const QFontMetrics fontMetrics = QFontMetrics(QFont());
-
-    QPointF pos = boundingRect.bottomLeft();
-    pos.rx() += 0.5 * (boundingRect.width() - fontMetrics.width(text));
-    pos.ry() += fontMetrics.height();
-
-    painter->drawText(pos, text);
-
-#endif // QT_VERSION
-
-    if(m_current)
-    {
-        painter->save();
-
-        painter->setCompositionMode(QPainter::CompositionMode_Multiply);
-        painter->fillRect(boundingRect, widget->palette().highlight());
-
-        painter->restore();
-    }
-}
-
-void ThumbnailItem::setCurrent(bool current)
-{
-    if(m_current != current)
-    {
-        m_current = current;
-
-        update();
-    }
-}
-
-void ThumbnailItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
-{
-    if(event->modifiers() == Qt::NoModifier
-            && (event->button() == Qt::LeftButton || event->button() == Qt::MidButton))
-    {
-        emit linkClicked(event->button() == Qt::MidButton, index() + 1);
-
-        event->accept();
-        return;
-    }
-
-    event->ignore();
-}
-
-void ThumbnailItem::mouseDoubleClickEvent(QGraphicsSceneMouseEvent*)
-{
-}
-
-void ThumbnailItem::mouseMoveEvent(QGraphicsSceneMouseEvent*)
-{
-}
-
-void ThumbnailItem::mouseReleaseEvent(QGraphicsSceneMouseEvent*)
-{
-}
-
-void ThumbnailItem::contextMenuEvent(QGraphicsSceneContextMenuEvent*)
-{
-}
-
-void ThumbnailItem::loadInteractiveElements()
-{
-    const qreal width = size().width() / 72.0 * 25.4;
-    const qreal height = size().height() / 72.0 * 25.4;
-
-    const qreal longEdge = qMax(width, height);
-    const qreal shortEdge = qMin(width, height);
-
-    QString paperSize;
-
-    if(qAbs(longEdge - 279.4) <= 1.0 && qAbs(shortEdge - 215.9) <= 1.0)
-    {
-        paperSize = QLatin1String(" (Letter)");
-    }
-    else
-    {
-        qreal longEdgeA = 1189.0;
-        qreal shortEdgeA = 841.0;
-
-        qreal longEdgeB = 1414.0;
-        qreal shortEdgeB = 1000.0;
-
-        for(int i = 0; i <= 10; ++i)
-        {
-            if(qAbs(longEdge - longEdgeA) <= 1.0 && qAbs(shortEdge - shortEdgeA) <= 1.0)
-            {
-                paperSize = QString(" (A%1)").arg(i);
-                break;
-            }
-            else if(qAbs(longEdge - longEdgeB) <= 1.0 && qAbs(shortEdge - shortEdgeB) <= 1.0)
-            {
-                paperSize = QString(" (B%1)").arg(i);
-                break;
-            }
-
-            longEdgeA = shortEdgeA;
-            shortEdgeA /= qSqrt(2.0);
-
-            longEdgeB = shortEdgeB;
-            shortEdgeB /= qSqrt(2.0);
-        }
-    }
-
-    setToolTip(QString("%1 mm x %2 mm%3").arg(width, 0, 'f', 1).arg(height, 0, 'f', 1).arg(paperSize));
-}
-
 } // qpdfview
