@@ -26,12 +26,6 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <QGraphicsObject>
 #include <QIcon>
 
-#if QT_VERSION >= QT_VERSION_CHECK(4,7,0)
-
-#include <QStaticText>
-
-#endif // QT_VERSION
-
 class QGraphicsProxyWidget;
 
 #include "global.h"
@@ -49,51 +43,28 @@ class Page;
 
 class Settings;
 class RenderTask;
+class TileItem;
 
 class PageItem : public QGraphicsObject
 {
     Q_OBJECT
 
+    friend class TileItem;
+
 public:
     PageItem(Model::Page* page, int index, bool presentationMode = false, QGraphicsItem* parent = 0);
     ~PageItem();
 
     QRectF boundingRect() const;
-    void paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*);
+    void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget*);
 
     inline int index() const { return m_index; }
+    inline bool presentationMode() const { return m_presentationMode; }
+
     inline const QSizeF& size() const { return m_size; }
 
-    inline qreal unscaledWidth() const
-    {
-        switch(m_rotation)
-        {
-        default:
-        case RotateBy0:
-        case RotateBy180:
-            return m_resolutionX / 72.0 * m_size.width();
-        case RotateBy90:
-        case RotateBy270:
-            return m_resolutionX / 72.0 * m_size.height();
-        }
-    }
-
-    inline qreal unscaledHeight() const
-    {
-        switch(m_rotation)
-        {
-        default:
-        case RotateBy0:
-        case RotateBy180:
-            return m_resolutionY / 72.0 * m_size.height();
-        case RotateBy90:
-        case RotateBy270:
-            return m_resolutionY / 72.0 * m_size.width();
-        }
-    }
-
-    inline bool invertColors() { return m_invertColors; }
-    void setInvertColors(bool invertColors);
+    qreal displayedWidth() const;
+    qreal displayedHeight() const;
 
     inline const QList< QRectF >& highlights() const { return m_highlights; }
     void setHighlights(const QList< QRectF >& highlights);
@@ -104,23 +75,28 @@ public:
     inline bool showsAnnotationOverlay() const { return !m_annotationOverlay.isEmpty(); }
     inline bool showsFormFieldOverlay() const { return !m_formFieldOverlay.isEmpty(); }
 
-    inline int resolutionX() const { return m_resolutionX; }
-    inline int resolutionY() const { return m_resolutionY; }
+    inline int resolutionX() const { return m_renderParam.resolution.resolutionX; }
+    inline int resolutionY() const { return m_renderParam.resolution.resolutionY; }
     void setResolution(int resolutionX, int resolutionY);
 
-    inline qreal devicePixelRatio() const { return m_devicePixelRatio; }
+    inline qreal devicePixelRatio() const { return m_renderParam.resolution.devicePixelRatio; }
     void setDevicePixelRatio(qreal devicePixelRatio);
 
-    inline qreal scaleFactor() const { return m_scaleFactor; }
+    inline qreal scaleFactor() const { return m_renderParam.scaleFactor; }
     void setScaleFactor(qreal scaleFactor);
 
-    inline Rotation rotation() const { return m_rotation; }
+    inline Rotation rotation() const { return m_renderParam.rotation; }
     void setRotation(Rotation rotation);
 
+    inline bool invertColors() { return m_renderParam.invertColors; }
+    void setInvertColors(bool invertColors);
+
     inline const QTransform& transform() const { return m_transform; }
     inline const QTransform& normalizedTransform() const { return m_normalizedTransform; }
 
 signals:
+    void cropRectChanged();
+
     void linkClicked(bool newTab, int page, qreal left = 0.0, qreal top = 0.0);
     void linkClicked(const QString& url);
     void linkClicked(const QString& fileName, int page);
@@ -134,15 +110,12 @@ signals:
     void wasModified();
 
 public slots:
-    void refresh();
+    void refresh(bool keepObsoletePixmaps = false, bool dropCachedPixmaps = false);
 
-    void startRender(bool prefetch = false);
+    int startRender(bool prefetch = false);
     void cancelRender();
 
 protected slots:
-    void on_renderTask_finished();
-    void on_renderTask_imageReady(int resolutionX, int resolutionY, qreal devicePixelRatio, qreal scaleFactor, Rotation rotation, bool invertColors, bool prefetch, QImage image);
-
     void showAnnotationOverlay(Model::Annotation* selectedAnnotation);
     void hideAnnotationOverlay(bool deleteLater = true);
     void updateAnnotationOverlay();
@@ -171,76 +144,66 @@ private:
 
     static Settings* s_settings;
 
-    static QCache< PageItem*, QPixmap > s_cache;
-
     Model::Page* m_page;
+    QSizeF m_size;
+
+    QRectF m_cropRect;
+
+    void updateCropRect();
 
     int m_index;
-    QSizeF m_size;
+    bool m_presentationMode;
+
+    QList< QRectF > m_highlights;
+
+    // interactive elements
 
     QList< Model::Link* > m_links;
     QList< Model::Annotation* > m_annotations;
     QList< Model::FormField* > m_formFields;
 
+    RubberBandMode m_rubberBandMode;
+    QRectF m_rubberBand;
+
+    void copyToClipboard(const QPoint& screenPos);
+    void addAnnotation(const QPoint& screenPos);
+
+    void showLinkContextMenu(Model::Link* link, const QPoint& screenPos);
+    void showAnnotationContextMenu(Model::Annotation* annotation, const QPoint& screenPos);
+
+    // overlay
+
     typedef QMap< Model::Annotation*, QGraphicsProxyWidget* > AnnotationOverlay;
     AnnotationOverlay m_annotationOverlay;
 
     typedef QMap< Model::FormField*, QGraphicsProxyWidget* > FormFieldOverlay;
     FormFieldOverlay m_formFieldOverlay;
 
-    bool m_presentationMode;
-    bool m_invertColors;
-
-    QList< QRectF > m_highlights;
-
-    RubberBandMode m_rubberBandMode;
-    QRectF m_rubberBand;
-
-    void copyToClipboard(const QPoint& screenPos);
-
-    void addAnnotation(const QPoint& screenPos);
-    void removeAnnotation(Model::Annotation* annotation, const QPoint& screenPos);
-
     template< typename Overlay, typename Element > void showOverlay(Overlay& overlay, const char* hideOverlay, const QList< Element* >& elements, Element* selectedElement);
     template< typename Overlay, typename Element > void addProxy(Overlay& overlay, const char* hideOverlay, Element* element);
     template< typename Overlay > void hideOverlay(Overlay& overlay, bool deleteLater);
     template< typename Overlay > void updateOverlay(const Overlay& overlay) const;
 
-    static const qreal proxyPadding = 2.0;
     void setProxyGeometry(Model::Annotation* annotation, QGraphicsProxyWidget* proxy) const;
     void setProxyGeometry(Model::FormField* formField, QGraphicsProxyWidget* proxy) const;
 
     // geometry
 
-    int m_resolutionX;
-    int m_resolutionY;
-    qreal m_devicePixelRatio;
-
-    qreal effectiveDevicePixelRatio();
-
-    qreal m_scaleFactor;
-    Rotation m_rotation;
+    RenderParam m_renderParam;
 
     QTransform m_transform;
     QTransform m_normalizedTransform;
     QRectF m_boundingRect;
 
-    QPixmap m_pixmap;
-
     void prepareGeometry();
 
-    RenderTask* m_renderTask;
+    QVector< TileItem* > m_tileItems;
 
-    // obsolete pixmap
-
-    QPixmap m_obsoletePixmap;
-    QPointF m_obsoleteTopLeft;
-    QTransform m_obsoleteTransform;
+    void prepareTiling();
 
     // paint
 
-    QPixmap cachedPixmap();
-    void paintPage(QPainter* painter, const QPixmap& pixmap) const;
+    void paintPage(QPainter* painter, const QRectF& exposedRect) const;
 
     void paintLinks(QPainter* painter) const;
     void paintFormFields(QPainter* painter) const;
@@ -250,43 +213,6 @@ private:
 
 };
 
-class ThumbnailItem : public PageItem
-{
-    Q_OBJECT
-
-public:
-    ThumbnailItem(Model::Page* page, int index, QGraphicsItem* parent = 0);
-
-    QRectF boundingRect() const;
-    void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget);
-
-    inline bool isCurrent() const { return m_current; }
-    void setCurrent(bool current);
-
-protected:
-    void mousePressEvent(QGraphicsSceneMouseEvent* event);
-    void mouseDoubleClickEvent(QGraphicsSceneMouseEvent*);
-    void mouseMoveEvent(QGraphicsSceneMouseEvent*);
-    void mouseReleaseEvent(QGraphicsSceneMouseEvent*);
-
-    void contextMenuEvent(QGraphicsSceneContextMenuEvent*);
-
-private slots:
-    void loadInteractiveElements();
-
-private:
-    Q_DISABLE_COPY(ThumbnailItem)
-
-#if QT_VERSION >= QT_VERSION_CHECK(4,7,0)
-
-    QStaticText m_text;
-
-#endif // QT_VERSION
-
-    bool m_current;
-
-};
-
 } // qpdfview
 
 #endif // PAGEITEM_H
