@@ -1,6 +1,7 @@
 /*
 
-Copyright 2013 Adam Reichold
+Copyright 2014 S. Razi Alavizadeh
+Copyright 2013-2014 Adam Reichold
 
 This file is part of qpdfview.
 
@@ -289,24 +290,6 @@ QImage PdfPage::render(qreal horizontalResolution, qreal verticalResolution, Rot
 
 #endif // HAS_POPPLER_24
 
-    double xres;
-    double yres;
-
-    switch(rotation)
-    {
-    default:
-    case RotateBy0:
-    case RotateBy180:
-        xres = horizontalResolution;
-        yres = verticalResolution;
-        break;
-    case RotateBy90:
-    case RotateBy270:
-        xres = verticalResolution;
-        yres = horizontalResolution;
-        break;
-    }
-
     Poppler::Page::Rotation rotate;
 
     switch(rotation)
@@ -339,7 +322,18 @@ QImage PdfPage::render(qreal horizontalResolution, qreal verticalResolution, Rot
         h = boundingRect.height();
     }
 
-    return m_page->renderToImage(xres, yres, x, y, w, h, rotate);
+    return m_page->renderToImage(horizontalResolution, verticalResolution, x, y, w, h, rotate);
+}
+
+QString PdfPage::label() const
+{
+#ifndef HAS_POPPLER_24
+
+    QMutexLocker mutexLocker(m_mutex);
+
+#endif // HAS_POPPLER_24
+
+    return m_page->label();
 }
 
 QList< Link* > PdfPage::links() const
@@ -450,7 +444,7 @@ QList< QRectF > PdfPage::search(const QString& text, bool matchCase) const
         results.append(rect);
     }
 
-#endif
+#endif // HAS_POPPLER_22 HAS_POPPLER_14
 
     return results;
 }
@@ -479,7 +473,7 @@ QList< Annotation* > PdfPage::annotations() const
     return annotations;
 }
 
-bool Model::PdfPage::canAddAndRemoveAnnotations() const
+bool PdfPage::canAddAndRemoveAnnotations() const
 {
 #ifdef HAS_POPPLER_20
 
@@ -807,7 +801,7 @@ void PdfDocument::loadProperties(QStandardItemModel* propertiesModel) const
 
 void PdfDocument::loadFonts(QStandardItemModel* fontsModel) const
 {
-    Document::Document::loadFonts(fontsModel);
+    Document::loadFonts(fontsModel);
 
 #ifndef HAS_POPPLER_24
 
@@ -1054,7 +1048,7 @@ Model::Document* PdfPlugin::loadDocument(const QString& filePath) const
         document->setRenderHint(Poppler::Document::Antialiasing, m_settings->value("antialiasing", true).toBool());
         document->setRenderHint(Poppler::Document::TextAntialiasing, m_settings->value("textAntialiasing", true).toBool());
 
-#ifdef HAS_POPPLER_18
+#if defined(HAS_POPPLER_18)
 
         switch(m_settings->value("textHinting", 0).toInt())
         {
@@ -1072,11 +1066,11 @@ Model::Document* PdfPlugin::loadDocument(const QString& filePath) const
             break;
         }
 
-#else
+#elif defined(HAS_POPPLER_14)
 
         document->setRenderHint(Poppler::Document::TextHinting, m_settings->value("textHinting", false).toBool());
 
-#endif // HAS_POPPLER_18
+#endif // HAS_POPPLER_18 HAS_POPPLER_14
 
 #ifdef HAS_POPPLER_22
 
