@@ -75,7 +75,10 @@ PresentationView::PresentationView(const QVector< Model::Page* >& pages, QWidget
     m_prefetchTimer->setSingleShot(true);
 
     connect(this, SIGNAL(currentPageChanged(int)), m_prefetchTimer, SLOT(start()));
+    connect(this, SIGNAL(scaleModeChanged(ScaleMode)), m_prefetchTimer, SLOT(start()));
+    connect(this, SIGNAL(scaleFactorChanged(qreal)), m_prefetchTimer, SLOT(start()));
     connect(this, SIGNAL(rotationChanged(Rotation)), m_prefetchTimer, SLOT(start()));
+    connect(this, SIGNAL(invertColorsChanged(bool)), m_prefetchTimer, SLOT(start()));
 
     connect(m_prefetchTimer, SIGNAL(timeout()), SLOT(on_prefetch_timeout()));
 
@@ -134,7 +137,9 @@ qreal PresentationView::scaleFactor() const
 
 void PresentationView::setScaleFactor(qreal scaleFactor)
 {
-    if(!qFuzzyCompare(m_scaleFactor, scaleFactor) && scaleFactor >= Defaults::DocumentView::minimumScaleFactor() && scaleFactor <= Defaults::DocumentView::maximumScaleFactor())
+    if(!qFuzzyCompare(m_scaleFactor, scaleFactor)
+            && scaleFactor >= s_settings->documentView().minimumScaleFactor()
+            && scaleFactor <= s_settings->documentView().maximumScaleFactor())
     {
         m_scaleFactor = scaleFactor;
 
@@ -256,12 +261,15 @@ void PresentationView::zoomIn()
 {
     if(scaleMode() != ScaleFactorMode)
     {
-        setScaleFactor(qMin(m_pageItems.at(m_currentPage - 1)->scaleFactor() + Defaults::DocumentView::zoomBy(), Defaults::DocumentView::maximumScaleFactor()));
+        setScaleFactor(qMin(m_pageItems.at(m_currentPage - 1)->scaleFactor() * s_settings->documentView().zoomFactor(),
+                            s_settings->documentView().maximumScaleFactor()));
+
         setScaleMode(ScaleFactorMode);
     }
     else
     {
-        setScaleFactor(qMin(m_scaleFactor + Defaults::DocumentView::zoomBy(), Defaults::DocumentView::maximumScaleFactor()));
+        setScaleFactor(qMin(m_scaleFactor * s_settings->documentView().zoomFactor(),
+                            s_settings->documentView().maximumScaleFactor()));
     }
 }
 
@@ -269,12 +277,15 @@ void PresentationView::zoomOut()
 {
     if(scaleMode() != ScaleFactorMode)
     {
-        setScaleFactor(qMax(m_pageItems.at(m_currentPage - 1)->scaleFactor() - Defaults::DocumentView::zoomBy(), Defaults::DocumentView::minimumScaleFactor()));
+        setScaleFactor(qMax(m_pageItems.at(m_currentPage - 1)->scaleFactor() / s_settings->documentView().zoomFactor(),
+                            s_settings->documentView().minimumScaleFactor()));
+
         setScaleMode(ScaleFactorMode);
     }
     else
     {
-        setScaleFactor(qMax(m_scaleFactor - Defaults::DocumentView::zoomBy(), Defaults::DocumentView::minimumScaleFactor()));
+        setScaleFactor(qMax(m_scaleFactor / s_settings->documentView().zoomFactor(),
+                            s_settings->documentView().minimumScaleFactor()));
     }
 }
 
@@ -334,14 +345,39 @@ void PresentationView::on_prefetch_timeout()
     fromPage = qMax(fromPage, 1);
     toPage = qMin(toPage, m_pages.count());
 
-    for(int index = fromPage - 1; index <= toPage - 1; ++index)
+    const int maxCost = toPage - fromPage + 1;
+    int cost = 0;
+
+    for(int index = m_currentPage - 1; index <= toPage - 1; ++index)
     {
-        m_pageItems.at(index)->startRender(true);
+        cost += m_pageItems.at(index)->startRender(true);
+
+        if(cost >= maxCost)
+        {
+            return;
+        }
+    }
+
+    for(int index = m_currentPage - 1; index >= fromPage - 1; --index)
+    {
+        cost += m_pageItems.at(index)->startRender(true);
+
+        if(cost >= maxCost)
+        {
+            return;
+        }
     }
 }
 
-void PresentationView::on_pages_linkClicked(int page, qreal left, qreal top)
+void PresentationView::on_pages_cropRectChanged()
 {
+    prepareScene();
+    prepareView();
+}
+
+void PresentationView::on_pages_linkClicked(bool newTab, int page, qreal left, qreal top)
+{
+    Q_UNUSED(newTab);
     Q_UNUSED(left);
     Q_UNUSED(top);
 
@@ -512,7 +548,9 @@ void PresentationView::preparePages()
         scene()->addItem(page);
         m_pageItems.append(page);
 
-        connect(page, SIGNAL(linkClicked(int,qreal,qreal)), SLOT(on_pages_linkClicked(int,qreal,qreal)));
+        connect(page, SIGNAL(cropRectChanged()), SLOT(on_pages_cropRectChanged()));
+
+        connect(page, SIGNAL(linkClicked(bool,int,qreal,qreal)), SLOT(on_pages_linkClicked(bool,int,qreal,qreal)));
     }
 }
 
@@ -535,10 +573,11 @@ void PresentationView::prepareBackground()
 
 void PresentationView::prepareScene()
 {
-    for(int index = 0; index < m_pageItems.count(); ++index)
-    {
-        PageItem* page = m_pageItems.at(index);
+    const qreal visibleWidth = viewport()->width();
+    const qreal visibleHeight = viewport()->height();
 
+    foreach(PageItem* page, m_pageItems)
+    {
 #if QT_VERSION >= QT_VERSION_CHECK(5,1,0)
 
         page->setDevicePixelRatio(devicePixelRatio());
@@ -547,26 +586,10 @@ void PresentationView::prepareScene()
 
         page->setResolution(logicalDpiX(), logicalDpiY());
 
-        const qreal visibleWidth = viewport()->width();
-        const qreal visibleHeight = viewport()->height();
+        page->setRotation(m_rotation);
 
-        qreal pageWidth = 0.0;
-        qreal pageHeight = 0.0;
-
-        switch(m_rotation)
-        {
-        default:
-        case RotateBy0:
-        case RotateBy180:
-            pageWidth = logicalDpiX() / 72.0 * page->size().width();
-            pageHeight = logicalDpiY() / 72.0 * page->size().height();
-            break;
-        case RotateBy90:
-        case RotateBy270:
-            pageWidth = logicalDpiX() / 72.0 * page->size().height();
-            pageHeight = logicalDpiY() / 72.0 * page->size().width();
-            break;
-        }
+        const qreal displayedWidth = page->displayedWidth();
+        const qreal displayedHeight = page->displayedHeight();
 
         switch(m_scaleMode)
         {
@@ -575,14 +598,12 @@ void PresentationView::prepareScene()
             page->setScaleFactor(m_scaleFactor);
             break;
         case FitToPageWidthMode:
-            page->setScaleFactor(visibleWidth / pageWidth);
+            page->setScaleFactor(visibleWidth / displayedWidth);
             break;
         case FitToPageSizeMode:
-            page->setScaleFactor(qMin(visibleWidth / pageWidth, visibleHeight / pageHeight));
+            page->setScaleFactor(qMin(visibleWidth / displayedWidth, visibleHeight / displayedHeight));
             break;
         }
-
-        page->setRotation(m_rotation);
     }
 }
 
