@@ -80,12 +80,17 @@ void Settings::PageItem::sync()
 {
     m_cacheSize = m_settings->value("pageItem/cacheSize", Defaults::PageItem::cacheSize()).toInt();
 
+    m_useTiling = m_settings->value("pageItem/useTiling", Defaults::PageItem::useTiling()).toBool();
+    m_tileSize = m_settings->value("pageItem/tileSize", Defaults::PageItem::tileSize()).toInt();
+
     m_progressIcon = QIcon::fromTheme("image-loading", QIcon(":/icons/image-loading.svg"));
     m_errorIcon = QIcon::fromTheme("image-missing", QIcon(":icons/image-missing.svg"));
 
     m_keepObsoletePixmaps = m_settings->value("pageItem/keepObsoletePixmaps", Defaults::PageItem::keepObsoletePixmaps()).toBool();
     m_useDevicePixelRatio = m_settings->value("pageItem/useDevicePixelRatio", Defaults::PageItem::useDevicePixelRatio()).toBool();
 
+    m_trimMargins = m_settings->value("pageItem/trimMargins", Defaults::PageItem::trimMargins()).toBool();
+
     m_decoratePages = m_settings->value("pageItem/decoratePages", Defaults::PageItem::decoratePages()).toBool();
     m_decorateLinks = m_settings->value("pageItem/decorateLinks", Defaults::PageItem::decorateLinks()).toBool();
     m_decorateFormFields = m_settings->value("pageItem/decorateFormFields", Defaults::PageItem::decorateFormFields()).toBool();
@@ -105,6 +110,12 @@ void Settings::PageItem::setCacheSize(int cacheSize)
     }
 }
 
+void Settings::PageItem::setUseTiling(bool useTiling)
+{
+    m_useTiling = useTiling;
+    m_settings->setValue("pageItem/useTiling", useTiling);
+}
+
 void Settings::PageItem::setKeepObsoletePixmaps(bool keepObsoletePixmaps)
 {
     m_keepObsoletePixmaps = keepObsoletePixmaps;
@@ -117,40 +128,46 @@ void Settings::PageItem::setUseDevicePixelRatio(bool useDevicePixelRatio)
     m_settings->setValue("pageItem/useDevicePixelRatio", useDevicePixelRatio);
 }
 
-void Settings::PageItem::setDecoratePages(bool decorate)
+void Settings::PageItem::setTrimMargins(bool trimMargins)
 {
-    m_decoratePages = decorate;
-    m_settings->setValue("pageItem/decoratePages", decorate);
+    m_trimMargins = trimMargins;
+    m_settings->setValue("pageItem/trimMargins", trimMargins);
 }
 
-void Settings::PageItem::setDecorateLinks(bool decorate)
+void Settings::PageItem::setDecoratePages(bool decoratePages)
 {
-    m_decorateLinks = decorate;
-    m_settings->setValue("pageItem/decorateLinks", decorate);
+    m_decoratePages = decoratePages;
+    m_settings->setValue("pageItem/decoratePages", decoratePages);
 }
 
-void Settings::PageItem::setDecorateFormFields(bool decorate)
+void Settings::PageItem::setDecorateLinks(bool decorateLinks)
 {
-    m_decorateFormFields = decorate;
-    m_settings->setValue("pageItem/decorateFormFields", decorate);
+    m_decorateLinks = decorateLinks;
+    m_settings->setValue("pageItem/decorateLinks", decorateLinks);
 }
 
-void Settings::PageItem::setBackgroundColor(const QColor &color)
+void Settings::PageItem::setDecorateFormFields(bool decorateFormFields)
 {
-    m_backgroundColor = color;
-    m_settings->setValue("pageItem/backgroundColor", color);
+    m_decorateFormFields = decorateFormFields;
+    m_settings->setValue("pageItem/decorateFormFields", decorateFormFields);
 }
 
-void Settings::PageItem::setPaperColor(const QColor& color)
+void Settings::PageItem::setBackgroundColor(const QColor& backgroundColor)
 {
-    m_paperColor = color;
-    m_settings->setValue("pageItem/paperColor", color);
+    m_backgroundColor = backgroundColor;
+    m_settings->setValue("pageItem/backgroundColor", backgroundColor);
 }
 
-void Settings::PageItem::setHighlightColor(const QColor& color)
+void Settings::PageItem::setPaperColor(const QColor& paperColor)
 {
-    m_highlightColor = color;
-    m_settings->setValue("pageItem/highlightColor", color);
+    m_paperColor = paperColor;
+    m_settings->setValue("pageItem/paperColor", paperColor);
+}
+
+void Settings::PageItem::setHighlightColor(const QColor& highlightColor)
+{
+    m_highlightColor = highlightColor;
+    m_settings->setValue("pageItem/highlightColor", highlightColor);
 }
 
 QColor Settings::PageItem::annotationColor() const
@@ -158,9 +175,9 @@ QColor Settings::PageItem::annotationColor() const
     return m_settings->value("pageItem/annotationColor", Defaults::PageItem::annotationColor()).value< QColor >();
 }
 
-void Settings::PageItem::setAnnotationColor(const QColor &color)
+void Settings::PageItem::setAnnotationColor(const QColor& annotationColor)
 {
-    m_settings->setValue("pageItem/annotationColor", color);
+    m_settings->setValue("pageItem/annotationColor", annotationColor);
 }
 
 Qt::KeyboardModifiers Settings::PageItem::copyToClipboardModifiers() const
@@ -209,6 +226,8 @@ Settings::PageItem::PageItem(QSettings* settings) :
     m_progressIcon(),
     m_errorIcon(),
     m_keepObsoletePixmaps(Defaults::PageItem::keepObsoletePixmaps()),
+    m_useDevicePixelRatio(false),
+    m_trimMargins(false),
     m_decoratePages(Defaults::PageItem::decoratePages()),
     m_decorateLinks(Defaults::PageItem::decorateLinks()),
     m_decorateFormFields(Defaults::PageItem::decorateFormFields()),
@@ -346,6 +365,26 @@ void Settings::DocumentView::setLimitThumbnailsToResults(bool limitThumbnailsToR
     m_settings->setValue("documentView/limitThumbnailsToResults", limitThumbnailsToResults);
 }
 
+qreal Settings::DocumentView::minimumScaleFactor() const
+{
+    return m_settings->value("documentView/minimumScaleFactor", Defaults::DocumentView::minimumScaleFactor()).toReal();
+}
+
+qreal Settings::DocumentView::maximumScaleFactor() const
+{
+    return m_settings->value("documentView/maximumScaleFactor", Defaults::DocumentView::maximumScaleFactor()).toReal();
+}
+
+qreal Settings::DocumentView::zoomFactor() const
+{
+    return m_settings->value("documentView/zoomFactor", Defaults::DocumentView::zoomFactor()).toReal();
+}
+
+void Settings::DocumentView::setZoomFactor(qreal zoomFactor)
+{
+    m_settings->setValue("documentView/zoomFactor", zoomFactor);
+}
+
 void Settings::DocumentView::setPageSpacing(qreal pageSpacing)
 {
     if(pageSpacing >= 0.0)
@@ -538,9 +577,9 @@ bool Settings::MainWindow::trackRecentlyUsed() const
     return m_settings->value("mainWindow/trackRecentlyUsed", Defaults::MainWindow::trackRecentlyUsed()).toBool();
 }
 
-void Settings::MainWindow::setTrackRecentlyUsed(bool on)
+void Settings::MainWindow::setTrackRecentlyUsed(bool trackRecentlyUsed)
 {
-    m_settings->setValue("mainWindow/trackRecentlyUsed", on);
+    m_settings->setValue("mainWindow/trackRecentlyUsed", trackRecentlyUsed);
 }
 
 int Settings::MainWindow::recentlyUsedCount() const
@@ -568,9 +607,9 @@ bool Settings::MainWindow::keepRecentlyClosed() const
     return m_settings->value("mainWindow/keepRecentlyClosed", Defaults::MainWindow::keepRecentlyClosed()).toBool();
 }
 
-void Settings::MainWindow::setKeepRecentlyClosed(bool on)
+void Settings::MainWindow::setKeepRecentlyClosed(bool keepRecentlyClosed)
 {
-    m_settings->setValue("mainWindow/keepRecentlyClosed", on);
+    m_settings->setValue("mainWindow/keepRecentlyClosed", keepRecentlyClosed);
 }
 
 int Settings::MainWindow::recentlyClosedCount() const
@@ -588,9 +627,9 @@ bool Settings::MainWindow::restoreTabs() const
     return m_settings->value("mainWindow/restoreTabs", Defaults::MainWindow::restoreTabs()).toBool();
 }
 
-void Settings::MainWindow::setRestoreTabs(bool on)
+void Settings::MainWindow::setRestoreTabs(bool restoreTabs)
 {
-    m_settings->setValue("mainWindow/restoreTabs", on);
+    m_settings->setValue("mainWindow/restoreTabs", restoreTabs);
 }
 
 bool Settings::MainWindow::restoreBookmarks() const
@@ -598,9 +637,9 @@ bool Settings::MainWindow::restoreBookmarks() const
     return m_settings->value("mainWindow/restoreBookmarks", Defaults::MainWindow::restoreBookmarks()).toBool();
 }
 
-void Settings::MainWindow::setRestoreBookmarks(bool on)
+void Settings::MainWindow::setRestoreBookmarks(bool restoreBookmarks)
 {
-    m_settings->setValue("mainWindow/restoreBookmarks", on);
+    m_settings->setValue("mainWindow/restoreBookmarks", restoreBookmarks);
 }
 
 bool Settings::MainWindow::restorePerFileSettings() const
@@ -608,9 +647,9 @@ bool Settings::MainWindow::restorePerFileSettings() const
     return m_settings->value("mainWindow/restorePerFileSettings", Defaults::MainWindow::restorePerFileSettings()).toBool();
 }
 
-void Settings::MainWindow::setRestorePerFileSettings(bool on)
+void Settings::MainWindow::setRestorePerFileSettings(bool restorePerFileSettings)
 {
-    m_settings->setValue("mainWindow/restorePerFileSettings", on);
+    m_settings->setValue("mainWindow/restorePerFileSettings", restorePerFileSettings);
 }
 
 int Settings::MainWindow::saveDatabaseInterval() const
@@ -618,6 +657,11 @@ int Settings::MainWindow::saveDatabaseInterval() const
     return m_settings->value("mainWindow/saveDatabaseInterval", Defaults::MainWindow::saveDatabaseInterval()).toInt();
 }
 
+void Settings::MainWindow::setSaveDatabaseInterval(int saveDatabaseInterval)
+{
+    m_settings->setValue("mainWindow/saveDatabaseInterval", saveDatabaseInterval);
+}
+
 int Settings::MainWindow::tabPosition() const
 {
     return m_settings->value("mainWindow/tabPosition", Defaults::MainWindow::tabPosition()).toInt();
@@ -686,7 +730,7 @@ void Settings::MainWindow::setCurrentPageInWindowTitle(bool currentPageInTabText
 
 bool Settings::MainWindow::instanceNameInWindowTitle() const
 {
-    return m_settings->value("mainWindow/instanceNameInWindowTitle", Defaults::MainWindow::instancfeNameInWindowTitle()).toBool();
+    return m_settings->value("mainWindow/instanceNameInWindowTitle", Defaults::MainWindow::instanceNameInWindowTitle()).toBool();
 }
 
 void Settings::MainWindow::setInstanceNameInWindowTitle(bool instanceNameInWindowTitle)
@@ -694,6 +738,16 @@ void Settings::MainWindow::setInstanceNameInWindowTitle(bool instanceNameInWindo
     m_settings->setValue("mainWindow/instanceNameInWindowTitle", instanceNameInWindowTitle);
 }
 
+bool Settings::MainWindow::usePageLabel() const
+{
+    return m_settings->value("mainWindow/usePageLabel", Defaults::MainWindow::usePageLabel()).toBool();
+}
+
+void Settings::MainWindow::setUsePageLabel(bool usePageLabel)
+{
+    m_settings->setValue("mainWindow/usePageLabel", usePageLabel);
+}
+
 bool Settings::MainWindow::synchronizeOutlineView() const
 {
     return m_settings->value("mainWindow/synchronizeOutlineView", Defaults::MainWindow::synchronizeOutlineView()).toBool();
@@ -734,6 +788,16 @@ void Settings::MainWindow::setViewToolBar(const QStringList& viewToolBar)
     m_settings->setValue("mainWindow/viewToolBar", trimmed(viewToolBar));
 }
 
+bool Settings::MainWindow::toggleToolAndMenuBarsWithFullscreen() const
+{
+    return m_settings->value("mainWindow/toggleToolAndMenuBarsWithFullscreen", Defaults::MainWindow::toggleToolAndMenuBarsWithFullscreen()).toBool();
+}
+
+void Settings::MainWindow::setToggleToolAndMenuBarsWithFullscreen(bool toggleToolAndMenuBarsWithFullscreen) const
+{
+    m_settings->setValue("mainWindow/toggleToolAndMenuBarsWithFullscreen", toggleToolAndMenuBarsWithFullscreen);
+}
+
 bool Settings::MainWindow::hasIconTheme() const
 {
     return m_settings->contains("mainWindow/iconTheme");
@@ -896,6 +960,8 @@ void Settings::PrintDialog::setFitToPage(bool fitToPage)
     m_settings->setValue("printDialog/fitToPage", fitToPage);
 }
 
+#if QT_VERSION < QT_VERSION_CHECK(5,2,0)
+
 PrintOptions::PageSet Settings::PrintDialog::pageSet() const
 {
     return static_cast< PrintOptions::PageSet >(m_settings->value("printDialog/pageSet", static_cast< uint >(Defaults::PrintDialog::pageSet())).toUInt());
@@ -926,6 +992,8 @@ void Settings::PrintDialog::setNumberUpLayout(PrintOptions::NumberUpLayout numbe
     m_settings->setValue("printDialog/numberUpLayout", static_cast< uint >(numberUpLayout));
 }
 
+#endif // QT_VERSION
+
 void Settings::sync()
 {
     m_settings->sync();
