@@ -55,6 +55,11 @@ public:
         inline int cacheSize() const { return m_cacheSize; }
         void setCacheSize(int cacheSize);
 
+        inline bool useTiling() const { return m_useTiling; }
+        void setUseTiling(bool useTiling);
+
+        inline int tileSize() const { return m_tileSize; }
+
         inline const QIcon& progressIcon() const { return m_progressIcon; }
         inline const QIcon& errorIcon() const { return m_errorIcon; }
 
@@ -64,26 +69,29 @@ public:
         inline bool useDevicePixelRatio() const { return m_useDevicePixelRatio; }
         void setUseDevicePixelRatio(bool useDevicePixelRatio);
 
+        inline bool trimMargins() const { return m_trimMargins; }
+        void setTrimMargins(bool trimMargins);
+
         inline bool decoratePages() const { return m_decoratePages; }
-        void setDecoratePages(bool decorate);
+        void setDecoratePages(bool decoratePages);
 
         inline bool decorateLinks() const { return m_decorateLinks; }
-        void setDecorateLinks(bool decorate);
+        void setDecorateLinks(bool decorateLinks);
 
         inline bool decorateFormFields() const { return m_decorateFormFields; }
-        void setDecorateFormFields(bool decorate);
+        void setDecorateFormFields(bool decorateFormFields);
 
         inline const QColor& backgroundColor() const { return m_backgroundColor; }
-        void setBackgroundColor(const QColor& color);
+        void setBackgroundColor(const QColor& backgroundColor);
 
         inline const QColor& paperColor() const { return m_paperColor; }
-        void setPaperColor(const QColor& color);
+        void setPaperColor(const QColor& paperColor);
 
         inline const QColor& highlightColor() const { return m_highlightColor; }
-        void setHighlightColor(const QColor& color);
+        void setHighlightColor(const QColor& highlightColor);
 
         QColor annotationColor() const;
-        void setAnnotationColor(const QColor& color);
+        void setAnnotationColor(const QColor& annotationColor);
 
         Qt::KeyboardModifiers copyToClipboardModifiers() const;
         void setCopyToClipboardModifiers(const Qt::KeyboardModifiers& modifiers);
@@ -105,12 +113,17 @@ public:
 
         int m_cacheSize;
 
+        bool m_useTiling;
+        int m_tileSize;
+
         QIcon m_progressIcon;
         QIcon m_errorIcon;
 
         bool m_keepObsoletePixmaps;
         bool m_useDevicePixelRatio;
 
+        bool m_trimMargins;
+
         bool m_decoratePages;
         bool m_decorateLinks;
         bool m_decorateFormFields;
@@ -176,6 +189,12 @@ public:
         inline bool limitThumbnailsToResults() const { return m_limitThumbnailsToResults; }
         void setLimitThumbnailsToResults(bool limitThumbnailsToResults);
 
+        qreal minimumScaleFactor() const;
+        qreal maximumScaleFactor() const;
+
+        qreal zoomFactor() const;
+        void setZoomFactor(qreal zoomFactor);
+
         inline qreal pageSpacing() const { return m_pageSpacing; }
         void setPageSpacing(qreal pageSpacing);
 
@@ -256,7 +275,7 @@ public:
     {
     public:
         bool trackRecentlyUsed() const;
-        void setTrackRecentlyUsed(bool on);
+        void setTrackRecentlyUsed(bool trackRecentlyUsed);
 
         int recentlyUsedCount() const;
         void setRecentlyUsedCount(int recentlyUsedCount);
@@ -265,21 +284,22 @@ public:
         void setRecentlyUsed(const QStringList& recentlyUsed);
 
         bool keepRecentlyClosed() const;
-        void setKeepRecentlyClosed(bool on);
+        void setKeepRecentlyClosed(bool keepRecentlyClosed);
 
         int recentlyClosedCount() const;
         void setRecentlyClosedCount(int recentlyClosedCount);
 
         bool restoreTabs() const;
-        void setRestoreTabs(bool on);
+        void setRestoreTabs(bool restoreTabs);
 
         bool restoreBookmarks() const;
-        void setRestoreBookmarks(bool on);
+        void setRestoreBookmarks(bool restoreBookmarks);
 
         bool restorePerFileSettings() const;
-        void setRestorePerFileSettings(bool on);
+        void setRestorePerFileSettings(bool restorePerFileSettings);
 
         int saveDatabaseInterval() const;
+        void setSaveDatabaseInterval(int saveDatabaseInterval);
 
         int tabPosition() const;
         void setTabPosition(int tabPosition);
@@ -302,6 +322,9 @@ public:
         bool instanceNameInWindowTitle() const;
         void setInstanceNameInWindowTitle(bool instanceNameInWindowTitle);
 
+        bool usePageLabel() const;
+        void setUsePageLabel(bool usePageLabel);
+
         bool synchronizeOutlineView() const;
         void setSynchronizeOutlineView(bool synchronizeOutlineView);
 
@@ -314,6 +337,9 @@ public:
         QStringList viewToolBar() const;
         void setViewToolBar(const QStringList& viewToolBar);
 
+        bool toggleToolAndMenuBarsWithFullscreen() const;
+        void setToggleToolAndMenuBarsWithFullscreen(bool toggleToolAndMenuBarsWithFullscreen) const;
+
         bool hasIconTheme() const;
         QString iconTheme() const;
 
@@ -372,6 +398,8 @@ public:
         bool fitToPage() const;
         void setFitToPage(bool fitToPage);
 
+#if QT_VERSION < QT_VERSION_CHECK(5,2,0)
+
         PrintOptions::PageSet pageSet() const;
         void setPageSet(PrintOptions::PageSet pageSet);
 
@@ -381,6 +409,8 @@ public:
         PrintOptions::NumberUpLayout numberUpLayout() const;
         void setNumberUpLayout(PrintOptions::NumberUpLayout numberUpLayout);
 
+#endif // QT_VERSION
+
     private:
         PrintDialog(QSettings* settings);
         friend class Settings;
@@ -423,9 +453,14 @@ public:
     public:
         static inline int cacheSize() { return 32 * 1024 * 1024; }
 
+        static inline bool useTiling() { return false; }
+        static inline int tileSize() { return 1024; }
+
         static inline bool keepObsoletePixmaps() { return false; }
         static inline bool useDevicePixelRatio() { return false; }
 
+        static inline bool trimMargins() { return false; }
+
         static inline bool decoratePages() { return true; }
         static inline bool decorateLinks() { return true; }
         static inline bool decorateFormFields() { return true; }
@@ -482,7 +517,7 @@ public:
         static inline qreal minimumScaleFactor() { return 0.1; }
         static inline qreal maximumScaleFactor() { return 10.0; }
 
-        static inline qreal zoomBy() { return 0.1; }
+        static inline qreal zoomFactor() { return 1.1; }
 
         static inline qreal pageSpacing() { return 5.0; }
         static inline qreal thumbnailSpacing() { return 3.0; }
@@ -491,7 +526,7 @@ public:
 
         static inline bool matchCase() { return false; }
 
-        static inline int highlightDuration() { return 5000; }
+        static inline int highlightDuration() { return 5 * 1000; }
         static inline QString sourceEditor() { return QString(); }
 
         static inline Qt::KeyboardModifiers zoomModifiers() { return Qt::ControlModifier; }
@@ -534,7 +569,7 @@ public:
         static inline bool restoreBookmarks() { return false; }
         static inline bool restorePerFileSettings() { return false; }
 
-        static inline int saveDatabaseInterval() { return 30000; }
+        static inline int saveDatabaseInterval() { return 5 * 60 * 1000; }
 
         static inline int tabPosition() { return 0; }
         static inline int tabVisibility() { return 0; }
@@ -546,7 +581,9 @@ public:
         static inline bool documentTitleAsTabTitle() { return true; }
 
         static inline bool currentPageInWindowTitle() { return false; }
-        static inline bool instancfeNameInWindowTitle() { return false; }
+        static inline bool instanceNameInWindowTitle() { return false; }
+
+        static inline bool usePageLabel() { return true; }
 
         static inline bool synchronizeOutlineView() { return false; }
 
@@ -554,6 +591,8 @@ public:
         static inline QStringList editToolBar() { return QStringList() << "currentPage" << "previousPage" << "nextPage"; }
         static inline QStringList viewToolBar() { return QStringList() << "scaleFactor" << "zoomIn" << "zoomOut"; }
 
+        static inline bool toggleToolAndMenuBarsWithFullscreen() { return false; }
+
         static QString path();
 
     private:
@@ -576,11 +615,15 @@ public:
 
         static inline bool fitToPage() { return false; }
 
+#if QT_VERSION < QT_VERSION_CHECK(5,2,0)
+
         static inline PrintOptions::PageSet pageSet() { return PrintOptions::AllPages; }
 
         static inline PrintOptions::NumberUp numberUp() { return PrintOptions::SinglePage; }
         static inline PrintOptions::NumberUpLayout numberUpLayout() { return PrintOptions::LeftRightTopBottom; }
 
+#endif // QT_VERSION
+
     private:
         PrintDialog() {}
 
