@@ -90,6 +90,8 @@ SettingsDialog::SettingsDialog(QWidget* parent) : QDialog(parent)
         s_settings = Settings::instance();
     }
 
+    setWindowTitle(tr("Settings") + QLatin1String(" - qpdfview"));
+
     m_graphicsTabWidget = new QTabWidget(this);
     m_graphicsTabWidget->addTab(new QWidget(this), tr("General"));
 
@@ -148,12 +150,19 @@ SettingsDialog::SettingsDialog(QWidget* parent) : QDialog(parent)
     layout()->addWidget(m_tabWidget);
     layout()->addWidget(m_dialogButtonBox);
 
+    resize(s_settings->mainWindow().settingsDialogSize(sizeHint()));
+
     createBehaviorTab();
     createGraphicsTab();
     createInterfaceTab();
     createModifiersTab();
 }
 
+SettingsDialog::~SettingsDialog()
+{
+    s_settings->mainWindow().setSettingsDialogSize(size());
+}
+
 void SettingsDialog::accept()
 {
     QDialog::accept();
@@ -258,11 +267,22 @@ void SettingsDialog::createBehaviorTab()
 
     m_behaviorLayout->addRow(tr("Restore per-file settings:"), m_restorePerFileSettingsCheckBox);
 
+    // save database interval
+
+    m_saveDatabaseInterval = new QSpinBox(this);
+    m_saveDatabaseInterval->setSuffix(tr(" min"));
+    m_saveDatabaseInterval->setRange(0, 60);
+    m_saveDatabaseInterval->setSpecialValueText(tr("Never"));
+    m_saveDatabaseInterval->setValue(s_settings->mainWindow().saveDatabaseInterval() / 1000 / 60);
+
+    m_behaviorLayout->addRow(tr("Save database interval:"), m_saveDatabaseInterval);
+
 #ifndef WITH_SQL
 
     m_restoreTabsCheckBox->setEnabled(false);
     m_restoreBookmarksCheckBox->setEnabled(false);
     m_restorePerFileSettingsCheckBox->setEnabled(false);
+    m_saveDatabaseInterval->setEnabled(false);
 
 #endif // WITH_SQL
 
@@ -282,10 +302,26 @@ void SettingsDialog::createBehaviorTab()
 
     m_behaviorLayout->addRow(tr("Presentation screen:"), m_presentationScreenSpinBox);
 
+    // synchronize outline view
+
+    m_synchronizeOutlineViewCheckBox = new QCheckBox(this);
+    m_synchronizeOutlineViewCheckBox->setChecked(s_settings->mainWindow().synchronizeOutlineView());
+
+    m_behaviorLayout->addRow(tr("Synchronize outline view:"), m_synchronizeOutlineViewCheckBox);
+
+    // zoom factor
+
+    m_zoomFactorSpinBox = new QDoubleSpinBox(this);
+    m_zoomFactorSpinBox->setRange(1.0, 2.0);
+    m_zoomFactorSpinBox->setSingleStep(0.05);
+    m_zoomFactorSpinBox->setValue(s_settings->documentView().zoomFactor());
+
+    m_behaviorLayout->addRow(tr("Zoom factor:"), m_zoomFactorSpinBox);
+
     // highlight duration
 
     m_highlightDurationSpinBox = new QSpinBox(this);
-    m_highlightDurationSpinBox->setSuffix(" ms");
+    m_highlightDurationSpinBox->setSuffix(tr(" ms"));
     m_highlightDurationSpinBox->setRange(0, 60000);
     m_highlightDurationSpinBox->setSingleStep(500);
     m_highlightDurationSpinBox->setSpecialValueText(tr("None"));
@@ -326,10 +362,15 @@ void SettingsDialog::acceptBehaivorTab()
     s_settings->mainWindow().setRestoreTabs(m_restoreTabsCheckBox->isChecked());
     s_settings->mainWindow().setRestoreBookmarks(m_restoreBookmarksCheckBox->isChecked());
     s_settings->mainWindow().setRestorePerFileSettings(m_restorePerFileSettingsCheckBox->isChecked());
+    s_settings->mainWindow().setSaveDatabaseInterval(m_saveDatabaseInterval->value() * 60 * 1000);
 
     s_settings->presentationView().setSynchronize(m_synchronizePresentationCheckBox->isChecked());
     s_settings->presentationView().setScreen(m_presentationScreenSpinBox->value());
 
+    s_settings->mainWindow().setSynchronizeOutlineView(m_synchronizeOutlineViewCheckBox->isChecked());
+
+    s_settings->documentView().setZoomFactor(m_zoomFactorSpinBox->value());
+
     s_settings->documentView().setHighlightDuration(m_highlightDurationSpinBox->value());
     s_settings->pageItem().setHighlightColor(getValidColorFromCurrentText(m_highlightColorComboBox, Defaults::PageItem::highlightColor()));
     s_settings->pageItem().setAnnotationColor(getValidColorFromCurrentText(m_annotationColorComboBox, Defaults::PageItem::annotationColor()));
@@ -349,10 +390,15 @@ void SettingsDialog::resetBehaviorTab()
     m_restoreTabsCheckBox->setChecked(Defaults::MainWindow::restoreTabs());
     m_restoreBookmarksCheckBox->setChecked(Defaults::MainWindow::restoreBookmarks());
     m_restorePerFileSettingsCheckBox->setChecked(Defaults::MainWindow::restorePerFileSettings());
+    m_saveDatabaseInterval->setValue(Defaults::MainWindow::saveDatabaseInterval());
 
     m_synchronizePresentationCheckBox->setChecked(Defaults::PresentationView::synchronize());
     m_presentationScreenSpinBox->setValue(Defaults::PresentationView::screen());
 
+    m_synchronizeOutlineViewCheckBox->setChecked(Defaults::MainWindow::synchronizeOutlineView());
+
+    m_zoomFactorSpinBox->setValue(Defaults::DocumentView::zoomFactor());
+
     m_highlightDurationSpinBox->setValue(Defaults::DocumentView::highlightDuration());
     setCurrentTextToColorName(m_highlightColorComboBox, Defaults::PageItem::highlightColor());
     setCurrentTextToColorName(m_annotationColorComboBox, Defaults::PageItem::annotationColor());
@@ -362,6 +408,13 @@ void SettingsDialog::resetBehaviorTab()
 
 void SettingsDialog::createGraphicsTab()
 {
+    // use tiling
+
+    m_useTilingCheckBox = new QCheckBox(this);
+    m_useTilingCheckBox->setChecked(s_settings->pageItem().useTiling());
+
+    m_graphicsLayout->addRow(tr("Use tiling:"), m_useTilingCheckBox);
+
     // keep obsolete pixmaps
 
     m_keepObsoletePixmapsCheckBox = new QCheckBox(this);
@@ -380,6 +433,13 @@ void SettingsDialog::createGraphicsTab()
 
 #endif // QT_VERSION
 
+    // trim margins
+
+    m_trimMarginsCheckBox = new QCheckBox(this);
+    m_trimMarginsCheckBox->setChecked(s_settings->pageItem().trimMargins());
+
+    m_graphicsLayout->addRow(tr("Trim margins:"), m_trimMarginsCheckBox);
+
     // decorate pages
 
     m_decoratePagesCheckBox = new QCheckBox(this);
@@ -430,7 +490,7 @@ void SettingsDialog::createGraphicsTab()
     // page spacing
 
     m_pageSpacingSpinBox = new QDoubleSpinBox(this);
-    m_pageSpacingSpinBox->setSuffix(" px");
+    m_pageSpacingSpinBox->setSuffix(tr(" px"));
     m_pageSpacingSpinBox->setRange(0.0, 25.0);
     m_pageSpacingSpinBox->setSingleStep(0.25);
     m_pageSpacingSpinBox->setValue(s_settings->documentView().pageSpacing());
@@ -440,7 +500,7 @@ void SettingsDialog::createGraphicsTab()
     // thumbnail spacing
 
     m_thumbnailSpacingSpinBox = new QDoubleSpinBox(this);
-    m_thumbnailSpacingSpinBox->setSuffix(" px");
+    m_thumbnailSpacingSpinBox->setSuffix(tr(" px"));
     m_thumbnailSpacingSpinBox->setRange(0.0, 25.0);
     m_thumbnailSpacingSpinBox->setSingleStep(0.25);
     m_thumbnailSpacingSpinBox->setValue(s_settings->documentView().thumbnailSpacing());
@@ -450,7 +510,7 @@ void SettingsDialog::createGraphicsTab()
     // thumbnail size
 
     m_thumbnailSizeSpinBox = new QDoubleSpinBox(this);
-    m_thumbnailSizeSpinBox->setSuffix(" px");
+    m_thumbnailSizeSpinBox->setSuffix(tr(" px"));
     m_thumbnailSizeSpinBox->setRange(30.0, 300.0);
     m_thumbnailSizeSpinBox->setSingleStep(10.0);
     m_thumbnailSizeSpinBox->setValue(s_settings->documentView().thumbnailSize());
@@ -470,7 +530,18 @@ void SettingsDialog::createGraphicsTab()
     m_cacheSizeComboBox->addItem(tr("%1 MB").arg(512), 512 * 1024 * 1024);
     m_cacheSizeComboBox->addItem(tr("%1 MB").arg(1024), 1073741823);
     m_cacheSizeComboBox->addItem(tr("%1 MB").arg(2048), 2147483647);
-    m_cacheSizeComboBox->setCurrentIndex(m_cacheSizeComboBox->findData(s_settings->pageItem().cacheSize()));
+
+    const int cacheSize = s_settings->pageItem().cacheSize();
+    int cacheSizeIndex = m_cacheSizeComboBox->findData(cacheSize);
+
+    if(cacheSizeIndex == -1)
+    {
+        m_cacheSizeComboBox->addItem(tr("%1 MB").arg(cacheSize / 1024 / 1024), cacheSize);
+
+        cacheSizeIndex = m_cacheSizeComboBox->count() - 1;
+    }
+
+    m_cacheSizeComboBox->setCurrentIndex(cacheSizeIndex);
 
     m_graphicsLayout->addRow(tr("Cache size:"), m_cacheSizeComboBox);
 
@@ -492,6 +563,7 @@ void SettingsDialog::createGraphicsTab()
 
 void SettingsDialog::acceptGraphicsTab()
 {
+    s_settings->pageItem().setUseTiling(m_useTilingCheckBox->isChecked());
     s_settings->pageItem().setKeepObsoletePixmaps(m_keepObsoletePixmapsCheckBox->isChecked());
 
 #if QT_VERSION >= QT_VERSION_CHECK(5,1,0)
@@ -500,6 +572,8 @@ void SettingsDialog::acceptGraphicsTab()
 
 #endif // QT_VERSION
 
+    s_settings->pageItem().setTrimMargins(m_trimMarginsCheckBox->isChecked());
+
     s_settings->pageItem().setDecoratePages(m_decoratePagesCheckBox->isChecked());
     s_settings->pageItem().setDecorateLinks(m_decorateLinksCheckBox->isChecked());
     s_settings->pageItem().setDecorateFormFields(m_decorateFormFieldsCheckBox->isChecked());
@@ -537,6 +611,7 @@ void SettingsDialog::acceptGraphicsTab()
 
 void SettingsDialog::resetGraphicsTab()
 {
+    m_useTilingCheckBox->setChecked(Defaults::PageItem::useTiling());
     m_keepObsoletePixmapsCheckBox->setChecked(Defaults::PageItem::keepObsoletePixmaps());
 
 #if QT_VERSION >= QT_VERSION_CHECK(5,1,0)
@@ -545,6 +620,8 @@ void SettingsDialog::resetGraphicsTab()
 
 #endif // QT_VERSION
 
+    m_trimMarginsCheckBox->setChecked(Defaults::PageItem::trimMargins());
+
     m_decoratePagesCheckBox->setChecked(Defaults::PageItem::decoratePages());
     m_decorateLinksCheckBox->setChecked(Defaults::PageItem::decorateLinks());
     m_decorateFormFieldsCheckBox->setChecked(Defaults::PageItem::decorateFormFields());
@@ -635,6 +712,13 @@ void SettingsDialog::createInterfaceTab()
 
     m_interfaceLayout->addRow(tr("Recently closed count:"), m_recentlyClosedCountSpinBox);
 
+    // toggle tool and menu bars with fullscreen
+
+    m_toggleToolAndMenuBarsWithFullscreenCheckBox = new QCheckBox(this);
+    m_toggleToolAndMenuBarsWithFullscreenCheckBox->setChecked(s_settings->mainWindow().toggleToolAndMenuBarsWithFullscreen());
+
+    m_interfaceLayout->addRow(tr("Toggle tool and menu bars with fullscreen:"), m_toggleToolAndMenuBarsWithFullscreenCheckBox);
+
     // file tool bar
 
     m_fileToolBarLineEdit = new QLineEdit(this);
@@ -659,6 +743,13 @@ void SettingsDialog::createInterfaceTab()
 
     m_interfaceLayout->addRow(tr("View tool bar:"), m_viewToolBarLineEdit);
 
+    // use page label
+
+    m_usePageLabelCheckBox = new QCheckBox(this);
+    m_usePageLabelCheckBox->setChecked(s_settings->mainWindow().usePageLabel());
+
+    m_interfaceLayout->addRow(tr("Use page label:"), m_usePageLabelCheckBox);
+
     // document title as tab title
 
     m_documentTitleAsTabTitleCheckBox = new QCheckBox(this);
@@ -680,13 +771,6 @@ void SettingsDialog::createInterfaceTab()
 
     m_interfaceLayout->addRow(tr("Instance name in window title:"), m_instanceNameInWindowTitleCheckBox);
 
-    // synchronize outline view
-
-    m_synchronizeOutlineViewCheckBox = new QCheckBox(this);
-    m_synchronizeOutlineViewCheckBox->setChecked(s_settings->mainWindow().synchronizeOutlineView());
-
-    m_interfaceLayout->addRow(tr("Synchronize outline view:"), m_synchronizeOutlineViewCheckBox);
-
     // highlight current thumbnail
 
     m_highlightCurrentThumbnailCheckBox = new QCheckBox(this);
@@ -727,17 +811,18 @@ void SettingsDialog::acceptInterfaceTab()
     s_settings->mainWindow().setRecentlyUsedCount(m_recentlyUsedCountSpinBox->value());
     s_settings->mainWindow().setRecentlyClosedCount(m_recentlyClosedCountSpinBox->value());
 
+    s_settings->mainWindow().setToggleToolAndMenuBarsWithFullscreen(m_toggleToolAndMenuBarsWithFullscreenCheckBox->isChecked());
+
     s_settings->mainWindow().setFileToolBar(m_fileToolBarLineEdit->text().split(",", QString::SkipEmptyParts));
     s_settings->mainWindow().setEditToolBar(m_editToolBarLineEdit->text().split(",", QString::SkipEmptyParts));
     s_settings->mainWindow().setViewToolBar(m_viewToolBarLineEdit->text().split(",", QString::SkipEmptyParts));
 
+    s_settings->mainWindow().setUsePageLabel(m_usePageLabelCheckBox->isChecked());
     s_settings->mainWindow().setDocumentTitleAsTabTitle(m_documentTitleAsTabTitleCheckBox->isChecked());
 
     s_settings->mainWindow().setCurrentPageInWindowTitle(m_currentPageInWindowTitleCheckBox->isChecked());
     s_settings->mainWindow().setInstanceNameInWindowTitle(m_instanceNameInWindowTitleCheckBox->isChecked());
 
-    s_settings->mainWindow().setSynchronizeOutlineView(m_synchronizeOutlineViewCheckBox->isChecked());
-
     s_settings->documentView().setHighlightCurrentThumbnail(m_highlightCurrentThumbnailCheckBox->isChecked());
     s_settings->documentView().setLimitThumbnailsToResults(m_limitThumbnailsToResultsCheckBox->isChecked());
 
@@ -755,16 +840,17 @@ void SettingsDialog::resetInterfaceTab()
 
     m_recentlyUsedCountSpinBox->setValue(Defaults::MainWindow::recentlyUsedCount());
 
+    m_toggleToolAndMenuBarsWithFullscreenCheckBox->setChecked(Defaults::MainWindow::toggleToolAndMenuBarsWithFullscreen());
+
     m_fileToolBarLineEdit->setText(Defaults::MainWindow::fileToolBar().join(","));
     m_editToolBarLineEdit->setText(Defaults::MainWindow::editToolBar().join(","));
     m_viewToolBarLineEdit->setText(Defaults::MainWindow::viewToolBar().join(","));
 
+    m_usePageLabelCheckBox->setChecked(Defaults::MainWindow::usePageLabel());
     m_documentTitleAsTabTitleCheckBox->setChecked(Defaults::MainWindow::documentTitleAsTabTitle());
 
     m_currentPageInWindowTitleCheckBox->setChecked(Defaults::MainWindow::currentPageInWindowTitle());
-    m_instanceNameInWindowTitleCheckBox->setChecked(Defaults::MainWindow::instancfeNameInWindowTitle());
-
-    m_synchronizeOutlineViewCheckBox->setChecked(Defaults::MainWindow::synchronizeOutlineView());
+    m_instanceNameInWindowTitleCheckBox->setChecked(Defaults::MainWindow::instanceNameInWindowTitle());
 
     m_highlightCurrentThumbnailCheckBox->setChecked(Defaults::DocumentView::highlightCurrentThumbnail());
     m_limitThumbnailsToResultsCheckBox->setChecked(Defaults::DocumentView::limitThumbnailsToResults());
@@ -845,6 +931,7 @@ void SettingsDialog::createModifiersComboBox(QComboBox*& comboBox, const Qt::Key
     comboBox->addItem(QShortcut::tr("Shift and Ctrl"), static_cast< int >(Qt::ShiftModifier | Qt::ControlModifier));
     comboBox->addItem(QShortcut::tr("Shift and Alt"), static_cast< int >(Qt::ShiftModifier | Qt::AltModifier));
     comboBox->addItem(QShortcut::tr("Ctrl and Alt"), static_cast< int >(Qt::ControlModifier | Qt::AltModifier));
+    comboBox->addItem(QShortcut::tr("Right mouse button"), static_cast< int >(Qt::RightButton));
 
     setCurrentIndexFromKeyboardModifiers(comboBox, modifiers);
 }
