@@ -47,6 +47,7 @@ class SettingsDialog : public QDialog
 
 public:
     SettingsDialog(QWidget* parent = 0);
+    ~SettingsDialog();
 
 public slots:
     void accept();
@@ -88,10 +89,15 @@ private:
     QCheckBox* m_restoreTabsCheckBox;
     QCheckBox* m_restoreBookmarksCheckBox;
     QCheckBox* m_restorePerFileSettingsCheckBox;
+    QSpinBox* m_saveDatabaseInterval;
 
     QCheckBox* m_synchronizePresentationCheckBox;
     QSpinBox* m_presentationScreenSpinBox;
 
+    QCheckBox* m_synchronizeOutlineViewCheckBox;
+
+    QDoubleSpinBox* m_zoomFactorSpinBox;
+
     QSpinBox* m_highlightDurationSpinBox;
     QComboBox* m_highlightColorComboBox;
     QComboBox* m_annotationColorComboBox;
@@ -104,6 +110,7 @@ private:
 
     // graphics
 
+    QCheckBox* m_useTilingCheckBox;
     QCheckBox* m_keepObsoletePixmapsCheckBox;
 
 #if QT_VERSION >= QT_VERSION_CHECK(5,1,0)
@@ -112,6 +119,8 @@ private:
 
 #endif // QT_VERSION
 
+    QCheckBox* m_trimMarginsCheckBox;
+
     QCheckBox* m_decoratePagesCheckBox;
     QCheckBox* m_decorateLinksCheckBox;
     QCheckBox* m_decorateFormFieldsCheckBox;
@@ -146,17 +155,18 @@ private:
     QSpinBox* m_recentlyUsedCountSpinBox;
     QSpinBox* m_recentlyClosedCountSpinBox;
 
+    QCheckBox* m_toggleToolAndMenuBarsWithFullscreenCheckBox;
+
     QLineEdit* m_fileToolBarLineEdit;
     QLineEdit* m_editToolBarLineEdit;
     QLineEdit* m_viewToolBarLineEdit;
 
+    QCheckBox* m_usePageLabelCheckBox;
     QCheckBox* m_documentTitleAsTabTitleCheckBox;
 
     QCheckBox* m_currentPageInWindowTitleCheckBox;
     QCheckBox* m_instanceNameInWindowTitleCheckBox;
 
-    QCheckBox* m_synchronizeOutlineViewCheckBox;
-
     QCheckBox* m_highlightCurrentThumbnailCheckBox;
     QCheckBox* m_limitThumbnailsToResultsCheckBox;
 
